# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["ComputerDragParams"]


class ComputerDragParams(TypedDict, total=False):
    tab_id: str

    x1: float

    x2: float

    y1: float

    y2: float



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Iterable, Optional
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .._types import SequenceNotStr
from .logprob_param import LogprobParam
from .message_param import MessageParam
from .summary_param import SummaryParam
from .action_drag_param import ActionDragParam
from .action_find_param import ActionFindParam
from .action_move_param import ActionMoveParam
from .action_type_param import ActionTypeParam
from .action_wait_param import ActionWaitParam
from .output_logs_param import OutputLogsParam
from .action_click_param import ActionClickParam
from .output_image_param import OutputImageParam
from .action_scroll_param import ActionScrollParam
from .action_keypress_param import ActionKeypressParam
from .action_open_page_param import ActionOpenPageParam
from .action_screenshot_param import ActionScreenshotParam
from .comparison_filter_param import ComparisonFilterParam
from .action_double_click_param import ActionDoubleClickParam
from .response_input_file_param import ResponseInputFileParam
from .response_input_text_param import ResponseInputTextParam
from .action_search_source_param import ActionSearchSourceParam
from .annotation_file_path_param import AnnotationFilePathParam
from .mcp_approval_request_param import McpApprovalRequestParam
from .response_input_image_param import ResponseInputImageParam
from .annotation_url_citation_param import AnnotationURLCitationParam
from .annotation_file_citation_param import AnnotationFileCitationParam
from .annotation_container_file_citation_param import AnnotationContainerFileCitationParam

__all__ = [
    "ResponseCreateParams",
    "InputUnionMember1",
    "InputUnionMember1EasyInputMessageParam",
    "InputUnionMember1EasyInputMessageParamContentUnionMember1",
    "InputUnionMember1Message",
    "InputUnionMember1MessageContent",
    "InputUnionMember1ResponseOutputMessageParam",
    "InputUnionMember1ResponseOutputMessageParamContent",
    "InputUnionMember1ResponseOutputMessageParamContentResponseOutputTextParam",
    "InputUnionMember1ResponseOutputMessageParamContentResponseOutputTextParamAnnotation",
    "InputUnionMember1ResponseOutputMessageParamContentResponseOutputRefusalParam",
    "InputUnionMember1ResponseFileSearchToolCallParam",
    "InputUnionMember1ResponseFileSearchToolCallParamResult",
    "InputUnionMember1ResponseComputerToolCallParam",
    "InputUnionMember1ResponseComputerToolCallParamAction",
    "InputUnionMember1ResponseComputerToolCallParamPendingSafetyCheck",
    "InputUnionMember1ComputerCallOutput",
    "InputUnionMember1ComputerCallOutputOutput",
    "InputUnionMember1ComputerCallOutputAcknowledgedSafetyCheck",
    "InputUnionMember1ResponseFunctionWebSearchParam",
    "InputUnionMember1ResponseFunctionWebSearchParamAction",
    "InputUnionMember1ResponseFunctionWebSearchParamActionActionSearch",
    "InputUnionMember1ResponseFunctionToolCallParam",
    "InputUnionMember1FunctionCallOutput",
    "InputUnionMember1FunctionCallOutputOutputUnionMember1",
    "InputUnionMember1FunctionCallOutputOutputUnionMember1ResponseInputTextContentParam",
    "InputUnionMember1FunctionCallOutputOutputUnionMember1ResponseInputImageContentParam",
    "InputUnionMember1FunctionCallOutputOutputUnionMember1ResponseInputFileContentParam",
    "InputUnionMember1ResponseReasoningItemParam",
    "InputUnionMember1ResponseReasoningItemParamContent",
    "InputUnionMember1ResponseCompactionItemParamParam",
    "InputUnionMember1ImageGenerationCall",
    "InputUnionMember1ResponseCodeInterpreterToolCallParam",
    "InputUnionMember1ResponseCodeInterpreterToolCallParamOutput",
    "InputUnionMember1LocalShellCall",
    "InputUnionMember1LocalShellCallAction",
    "InputUnionMember1LocalShellCallOutput",
    "InputUnionMember1ShellCall",
    "InputUnionMember1ShellCallAction",
    "InputUnionMember1ShellCallOutput",
    "InputUnionMember1ShellCallOutputOutput",
    "InputUnionMember1ShellCallOutputOutputOutcome",
    "InputUnionMember1ShellCallOutputOutputOutcomeOutcomeTimeout",
    "InputUnionMember1ShellCallOutputOutputOutcomeOutcomeExit",
    "InputUnionMember1ApplyPatchCall",
    "InputUnionMember1ApplyPatchCallOperation",
    "InputUnionMember1ApplyPatchCallOperationApplyPatchCallOperationCreateFile",
    "InputUnionMember1ApplyPatchCallOperationApplyPatchCallOperationDeleteFile",
    "InputUnionMember1ApplyPatchCallOperationApplyPatchCallOperationUpdateFile",
    "InputUnionMember1ApplyPatchCallOutput",
    "InputUnionMember1McpListTools",
    "InputUnionMember1McpListToolsTool",
    "InputUnionMember1McpApprovalResponse",
    "InputUnionMember1McpCall",
    "InputUnionMember1ResponseCustomToolCallOutputParam",
    "InputUnionMember1ResponseCustomToolCallOutputParamOutputUnionMember1",
    "InputUnionMember1ResponseCustomToolCallParam",
    "InputUnionMember1ItemReference",
    "InputUnionMember1ResponseOutputMessage",
    "InputUnionMember1ResponseOutputMessageContent",
    "InputUnionMember1ResponseOutputMessageContentResponseOutputText",
    "InputUnionMember1ResponseOutputMessageContentResponseOutputTextAnnotation",
    "InputUnionMember1ResponseOutputMessageContentResponseOutputRefusal",
    "InputUnionMember1ResponseFileSearchToolCall",
    "InputUnionMember1ResponseFileSearchToolCallResult",
    "InputUnionMember1ResponseFunctionToolCall",
    "InputUnionMember1ResponseFunctionWebSearch",
    "InputUnionMember1ResponseFunctionWebSearchAction",
    "InputUnionMember1ResponseFunctionWebSearchActionActionSearch",
    "InputUnionMember1ResponseComputerToolCall",
    "InputUnionMember1ResponseComputerToolCallAction",
    "InputUnionMember1ResponseComputerToolCallPendingSafetyCheck",
    "InputUnionMember1ResponseReasoningItem",
    "InputUnionMember1ResponseReasoningItemContent",
    "InputUnionMember1ResponseCompactionItem",
    "InputUnionMember1ResponseCodeInterpreterToolCall",
    "InputUnionMember1ResponseCodeInterpreterToolCallOutput",
    "InputUnionMember1ResponseFunctionShellToolCall",
    "InputUnionMember1ResponseFunctionShellToolCallAction",
    "InputUnionMember1ResponseFunctionShellToolCallOutput",
    "InputUnionMember1ResponseFunctionShellToolCallOutputOutput",
    "InputUnionMember1ResponseFunctionShellToolCallOutputOutputOutcome",
    "InputUnionMember1ResponseFunctionShellToolCallOutputOutputOutcomeOutputOutcomeTimeout",
    "InputUnionMember1ResponseFunctionShellToolCallOutputOutputOutcomeOutputOutcomeExit",
    "InputUnionMember1ResponseApplyPatchToolCall",
    "InputUnionMember1ResponseApplyPatchToolCallOperation",
    "InputUnionMember1ResponseApplyPatchToolCallOperationOperationCreateFile",
    "InputUnionMember1ResponseApplyPatchToolCallOperationOperationDeleteFile",
    "InputUnionMember1ResponseApplyPatchToolCallOperationOperationUpdateFile",
    "InputUnionMember1ResponseApplyPatchToolCallOutput",
    "InputUnionMember1ResponseCustomToolCall",
    "PreviousInputMessage",
    "Prompt",
    "PromptVariables",
    "PromptVariablesResponseInputText",
    "PromptVariablesResponseInputImage",
    "PromptVariablesResponseInputFile",
    "Reasoning",
    "Text",
    "TextFormat",
    "TextFormatResponseFormatText",
    "TextFormatResponseFormatTextJsonSchemaConfig",
    "TextFormatResponseFormatJsonObject",
    "ToolChoice",
    "ToolChoiceToolChoiceAllowed",
    "ToolChoiceToolChoiceTypes",
    "ToolChoiceToolChoiceFunction",
    "ToolChoiceToolChoiceMcp",
    "ToolChoiceToolChoiceCustom",
    "ToolChoiceToolChoiceApplyPatch",
    "ToolChoiceToolChoiceShell",
    "Tool",
    "ToolFunctionTool",
    "ToolFileSearchTool",
    "ToolFileSearchToolFilters",
    "ToolFileSearchToolFiltersCompoundFilter",
    "ToolFileSearchToolFiltersCompoundFilterFilter",
    "ToolFileSearchToolRankingOptions",
    "ToolFileSearchToolRankingOptionsHybridSearch",
    "ToolComputerTool",
    "ToolWebSearchTool",
    "ToolWebSearchToolFilters",
    "ToolWebSearchToolUserLocation",
    "ToolMcp",
    "ToolMcpAllowedTools",
    "ToolMcpAllowedToolsMcpAllowedToolsMcpToolFilter",
    "ToolMcpRequireApproval",
    "ToolMcpRequireApprovalMcpRequireApprovalMcpToolApprovalFilter",
    "ToolMcpRequireApprovalMcpRequireApprovalMcpToolApprovalFilterAlways",
    "ToolMcpRequireApprovalMcpRequireApprovalMcpToolApprovalFilterNever",
    "ToolCodeInterpreter",
    "ToolCodeInterpreterContainer",
    "ToolCodeInterpreterContainerCodeInterpreterContainerCodeInterpreterToolAuto",
    "ToolImageGeneration",
    "ToolImageGenerationInputImageMask",
    "ToolLocalShell",
    "ToolFunctionShellTool",
    "ToolCustomTool",
    "ToolCustomToolFormat",
    "ToolCustomToolFormatText",
    "ToolCustomToolFormatGrammar",
    "ToolWebSearchPreviewTool",
    "ToolWebSearchPreviewToolUserLocation",
    "ToolApplyPatchTool",
]


class ResponseCreateParams(TypedDict, total=False):
    input: Required[Union[str, Iterable[InputUnionMember1]]]

    background: Optional[bool]

    cache_salt: Optional[str]
    """
    If specified, the prefix cache will be salted with the provided string to
    prevent an attacker to guess prompts in multi-user environments. The salt should
    be random, protected from access by 3rd parties, and long enough to be
    unpredictable (e.g., 43 characters base64-encoded, corresponding to 256 bit).
    """

    enable_response_messages: bool
    """Dictates whether or not to return messages as part of the response object.

    Currently only supported fornon-background and gpt-oss only.
    """

    include: Optional[
        List[
            Literal[
                "code_interpreter_call.outputs",
                "computer_call_output.output.image_url",
                "file_search_call.results",
                "message.input_image.image_url",
                "message.output_text.logprobs",
                "reasoning.encrypted_content",
            ]
        ]
    ]

    include_stop_str_in_output: bool

    instructions: Optional[str]

    logit_bias: Optional[Dict[str, float]]

    max_output_tokens: Optional[int]

    max_tool_calls: Optional[int]

    metadata: Optional[Dict[str, str]]

    mm_processor_kwargs: Optional[Dict[str, object]]
    """Additional kwargs to pass to the HF processor."""

    model: Optional[str]

    parallel_tool_calls: Optional[bool]

    previous_input_messages: Optional[Iterable[PreviousInputMessage]]

    previous_response_id: Optional[str]

    priority: int
    """The priority of the request (lower means earlier handling; default: 0).

    Any priority other than 0 will raise an error if the served model does not use
    priority scheduling.
    """

    prompt: Optional[Prompt]
    """
    Reference to a prompt template and its variables.
    [Learn more](https://platform.openai.com/docs/guides/text?api-mode=responses#reusable-prompts).
    """

    prompt_cache_key: Optional[str]
    """
    A key that was used to read from or write to the prompt cache.Note: This field
    has not been implemented yet and vLLM will ignore it.
    """

    reasoning: Optional[Reasoning]
    """**gpt-5 and o-series models only**

    Configuration options for
    [reasoning models](https://platform.openai.com/docs/guides/reasoning).
    """

    request_id: str
    """The request_id related to this request.

    If the caller does not set it, a random_uuid will be generated. This id is used
    through out the inference process and return in response.
    """

    service_tier: Literal["auto", "default", "flex", "scale", "priority"]

    skip_special_tokens: bool

    store: Optional[bool]

    stream: Optional[bool]

    temperature: Optional[float]

    text: Optional[Text]
    """Configuration options for a text response from the model.

    Can be plain text or structured JSON data. Learn more:

    - [Text inputs and outputs](https://platform.openai.com/docs/guides/text)
    - [Structured Outputs](https://platform.openai.com/docs/guides/structured-outputs)
    """

    tool_choice: ToolChoice
    """Constrains the tools available to the model to a pre-defined set."""

    tools: Iterable[Tool]

    top_k: Optional[int]

    top_logprobs: Optional[int]

    top_p: Optional[float]

    truncation: Optional[Literal["auto", "disabled"]]

    user: Optional[str]


InputUnionMember1EasyInputMessageParamContentUnionMember1: TypeAlias = Union[
    ResponseInputTextParam, ResponseInputImageParam, ResponseInputFileParam
]


class InputUnionMember1EasyInputMessageParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """
    A message input to the model with a role indicating instruction following
    hierarchy. Instructions given with the `developer` or `system` role take
    precedence over instructions given with the `user` role. Messages with the
    `assistant` role are presumed to have been generated by the model in previous
    interactions.
    """

    content: Required[Union[str, Iterable[InputUnionMember1EasyInputMessageParamContentUnionMember1]]]

    role: Required[Literal["user", "assistant", "system", "developer"]]

    type: Literal["message"]


InputUnionMember1MessageContent: TypeAlias = Union[
    ResponseInputTextParam, ResponseInputImageParam, ResponseInputFileParam
]


class InputUnionMember1Message(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """
    A message input to the model with a role indicating instruction following
    hierarchy. Instructions given with the `developer` or `system` role take
    precedence over instructions given with the `user` role.
    """

    content: Required[Iterable[InputUnionMember1MessageContent]]

    role: Required[Literal["user", "system", "developer"]]

    status: Literal["in_progress", "completed", "incomplete"]

    type: Literal["message"]


InputUnionMember1ResponseOutputMessageParamContentResponseOutputTextParamAnnotation: TypeAlias = Union[
    AnnotationFileCitationParam,
    AnnotationURLCitationParam,
    AnnotationContainerFileCitationParam,
    AnnotationFilePathParam,
]


class InputUnionMember1ResponseOutputMessageParamContentResponseOutputTextParam(  # type: ignore[call-arg]
    TypedDict, total=False, extra_items=object
):
    """A text output from the model."""

    annotations: Required[Iterable[InputUnionMember1ResponseOutputMessageParamContentResponseOutputTextParamAnnotation]]

    text: Required[str]

    type: Required[Literal["output_text"]]

    logprobs: Iterable[LogprobParam]


class InputUnionMember1ResponseOutputMessageParamContentResponseOutputRefusalParam(  # type: ignore[call-arg]
    TypedDict, total=False, extra_items=object
):
    """A refusal from the model."""

    refusal: Required[str]

    type: Required[Literal["refusal"]]


InputUnionMember1ResponseOutputMessageParamContent: TypeAlias = Union[
    InputUnionMember1ResponseOutputMessageParamContentResponseOutputTextParam,
    InputUnionMember1ResponseOutputMessageParamContentResponseOutputRefusalParam,
]


class InputUnionMember1ResponseOutputMessageParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """An output message from the model."""

    id: Required[str]

    content: Required[Iterable[InputUnionMember1ResponseOutputMessageParamContent]]

    role: Required[Literal["assistant"]]

    status: Required[Literal["in_progress", "completed", "incomplete"]]

    type: Required[Literal["message"]]


class InputUnionMember1ResponseFileSearchToolCallParamResult(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    attributes: Optional[Dict[str, Union[str, float, bool]]]

    file_id: str

    filename: str

    score: float

    text: str


class InputUnionMember1ResponseFileSearchToolCallParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """The results of a file search tool call.

    See the
    [file search guide](https://platform.openai.com/docs/guides/tools-file-search) for more information.
    """

    id: Required[str]

    queries: Required[SequenceNotStr[str]]

    status: Required[Literal["in_progress", "searching", "completed", "incomplete", "failed"]]

    type: Required[Literal["file_search_call"]]

    results: Optional[Iterable[InputUnionMember1ResponseFileSearchToolCallParamResult]]


InputUnionMember1ResponseComputerToolCallParamAction: TypeAlias = Union[
    ActionClickParam,
    ActionDoubleClickParam,
    ActionDragParam,
    ActionKeypressParam,
    ActionMoveParam,
    ActionScreenshotParam,
    ActionScrollParam,
    ActionTypeParam,
    ActionWaitParam,
]


class InputUnionMember1ResponseComputerToolCallParamPendingSafetyCheck(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A pending safety check for the computer call."""

    id: Required[str]

    code: Optional[str]

    message: Optional[str]


class InputUnionMember1ResponseComputerToolCallParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A tool call to a computer use tool.

    See the
    [computer use guide](https://platform.openai.com/docs/guides/tools-computer-use) for more information.
    """

    id: Required[str]

    action: Required[InputUnionMember1ResponseComputerToolCallParamAction]
    """A click action."""

    call_id: Required[str]

    pending_safety_checks: Required[Iterable[InputUnionMember1ResponseComputerToolCallParamPendingSafetyCheck]]

    status: Required[Literal["in_progress", "completed", "incomplete"]]

    type: Required[Literal["computer_call"]]


class InputUnionMember1ComputerCallOutputOutput(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A computer screenshot image used with the computer use tool."""

    type: Required[Literal["computer_screenshot"]]

    file_id: str

    image_url: str


class InputUnionMember1ComputerCallOutputAcknowledgedSafetyCheck(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A pending safety check for the computer call."""

    id: Required[str]

    code: Optional[str]

    message: Optional[str]


class InputUnionMember1ComputerCallOutput(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """The output of a computer tool call."""

    call_id: Required[str]

    output: Required[InputUnionMember1ComputerCallOutputOutput]
    """A computer screenshot image used with the computer use tool."""

    type: Required[Literal["computer_call_output"]]

    id: Optional[str]

    acknowledged_safety_checks: Optional[Iterable[InputUnionMember1ComputerCallOutputAcknowledgedSafetyCheck]]

    status: Optional[Literal["in_progress", "completed", "incomplete"]]


class InputUnionMember1ResponseFunctionWebSearchParamActionActionSearch(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """Action type "search" - Performs a web search query."""

    query: Required[str]

    type: Required[Literal["search"]]

    queries: SequenceNotStr[str]

    sources: Iterable[ActionSearchSourceParam]


InputUnionMember1ResponseFunctionWebSearchParamAction: TypeAlias = Union[
    InputUnionMember1ResponseFunctionWebSearchParamActionActionSearch, ActionOpenPageParam, ActionFindParam
]


class InputUnionMember1ResponseFunctionWebSearchParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """The results of a web search tool call.

    See the
    [web search guide](https://platform.openai.com/docs/guides/tools-web-search) for more information.
    """

    id: Required[str]

    action: Required[InputUnionMember1ResponseFunctionWebSearchParamAction]
    """Action type "search" - Performs a web search query."""

    status: Required[Literal["in_progress", "searching", "completed", "failed"]]

    type: Required[Literal["web_search_call"]]


class InputUnionMember1ResponseFunctionToolCallParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A tool call to run a function.

    See the
    [function calling guide](https://platform.openai.com/docs/guides/function-calling) for more information.
    """

    arguments: Required[str]

    call_id: Required[str]

    name: Required[str]

    type: Required[Literal["function_call"]]

    id: str

    status: Literal["in_progress", "completed", "incomplete"]


class InputUnionMember1FunctionCallOutputOutputUnionMember1ResponseInputTextContentParam(  # type: ignore[call-arg]
    TypedDict, total=False, extra_items=object
):
    """A text input to the model."""

    text: Required[str]

    type: Required[Literal["input_text"]]


class InputUnionMember1FunctionCallOutputOutputUnionMember1ResponseInputImageContentParam(  # type: ignore[call-arg]
    TypedDict, total=False, extra_items=object
):
    """An image input to the model.

    Learn about [image inputs](https://platform.openai.com/docs/guides/vision)
    """

    type: Required[Literal["input_image"]]

    detail: Optional[Literal["low", "high", "auto"]]

    file_id: Optional[str]

    image_url: Optional[str]


class InputUnionMember1FunctionCallOutputOutputUnionMember1ResponseInputFileContentParam(  # type: ignore[call-arg]
    TypedDict, total=False, extra_items=object
):
    """A file input to the model."""

    type: Required[Literal["input_file"]]

    file_data: Optional[str]

    file_id: Optional[str]

    file_url: Optional[str]

    filename: Optional[str]


InputUnionMember1FunctionCallOutputOutputUnionMember1: TypeAlias = Union[
    InputUnionMember1FunctionCallOutputOutputUnionMember1ResponseInputTextContentParam,
    InputUnionMember1FunctionCallOutputOutputUnionMember1ResponseInputImageContentParam,
    InputUnionMember1FunctionCallOutputOutputUnionMember1ResponseInputFileContentParam,
]


class InputUnionMember1FunctionCallOutput(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """The output of a function tool call."""

    call_id: Required[str]

    output: Required[Union[str, Iterable[InputUnionMember1FunctionCallOutputOutputUnionMember1]]]

    type: Required[Literal["function_call_output"]]

    id: Optional[str]

    status: Optional[Literal["in_progress", "completed", "incomplete"]]


class InputUnionMember1ResponseReasoningItemParamContent(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """Reasoning text from the model."""

    text: Required[str]

    type: Required[Literal["reasoning_text"]]


class InputUnionMember1ResponseReasoningItemParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """
    A description of the chain of thought used by a reasoning model while generating
    a response. Be sure to include these items in your `input` to the Responses API
    for subsequent turns of a conversation if you are manually
    [managing context](https://platform.openai.com/docs/guides/conversation-state).
    """

    id: Required[str]

    summary: Required[Iterable[SummaryParam]]

    type: Required[Literal["reasoning"]]

    content: Iterable[InputUnionMember1ResponseReasoningItemParamContent]

    encrypted_content: Optional[str]

    status: Literal["in_progress", "completed", "incomplete"]


class InputUnionMember1ResponseCompactionItemParamParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """
    A compaction item generated by the [`v1/responses/compact` API](https://platform.openai.com/docs/api-reference/responses/compact).
    """

    encrypted_content: Required[str]

    type: Required[Literal["compaction"]]

    id: Optional[str]


class InputUnionMember1ImageGenerationCall(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """An image generation request made by the model."""

    id: Required[str]

    result: Required[Optional[str]]

    status: Required[Literal["in_progress", "completed", "generating", "failed"]]

    type: Required[Literal["image_generation_call"]]


InputUnionMember1ResponseCodeInterpreterToolCallParamOutput: TypeAlias = Union[OutputLogsParam, OutputImageParam]


class InputUnionMember1ResponseCodeInterpreterToolCallParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A tool call to run code."""

    id: Required[str]

    code: Required[Optional[str]]

    container_id: Required[str]

    outputs: Required[Optional[Iterable[InputUnionMember1ResponseCodeInterpreterToolCallParamOutput]]]

    status: Required[Literal["in_progress", "completed", "incomplete", "interpreting", "failed"]]

    type: Required[Literal["code_interpreter_call"]]


class InputUnionMember1LocalShellCallAction(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """Execute a shell command on the server."""

    command: Required[SequenceNotStr[str]]

    env: Required[Dict[str, str]]

    type: Required[Literal["exec"]]

    timeout_ms: Optional[int]

    user: Optional[str]

    working_directory: Optional[str]


class InputUnionMember1LocalShellCall(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A tool call to run a command on the local shell."""

    id: Required[str]

    action: Required[InputUnionMember1LocalShellCallAction]
    """Execute a shell command on the server."""

    call_id: Required[str]

    status: Required[Literal["in_progress", "completed", "incomplete"]]

    type: Required[Literal["local_shell_call"]]


class InputUnionMember1LocalShellCallOutput(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """The output of a local shell tool call."""

    id: Required[str]

    output: Required[str]

    type: Required[Literal["local_shell_call_output"]]

    status: Optional[Literal["in_progress", "completed", "incomplete"]]


class InputUnionMember1ShellCallAction(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """The shell commands and limits that describe how to run the tool call."""

    commands: Required[SequenceNotStr[str]]

    max_output_length: Optional[int]

    timeout_ms: Optional[int]


class InputUnionMember1ShellCall(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A tool representing a request to execute one or more shell commands."""

    action: Required[InputUnionMember1ShellCallAction]
    """The shell commands and limits that describe how to run the tool call."""

    call_id: Required[str]

    type: Required[Literal["shell_call"]]

    id: Optional[str]

    status: Optional[Literal["in_progress", "completed", "incomplete"]]


class InputUnionMember1ShellCallOutputOutputOutcomeOutcomeTimeout(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """Indicates that the shell call exceeded its configured time limit."""

    type: Required[Literal["timeout"]]


class InputUnionMember1ShellCallOutputOutputOutcomeOutcomeExit(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """Indicates that the shell commands finished and returned an exit code."""

    exit_code: Required[int]

    type: Required[Literal["exit"]]


InputUnionMember1ShellCallOutputOutputOutcome: TypeAlias = Union[
    InputUnionMember1ShellCallOutputOutputOutcomeOutcomeTimeout,
    InputUnionMember1ShellCallOutputOutputOutcomeOutcomeExit,
]


class InputUnionMember1ShellCallOutputOutput(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """Captured stdout and stderr for a portion of a shell tool call output."""

    outcome: Required[InputUnionMember1ShellCallOutputOutputOutcome]
    """Indicates that the shell call exceeded its configured time limit."""

    stderr: Required[str]

    stdout: Required[str]


class InputUnionMember1ShellCallOutput(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """The streamed output items emitted by a shell tool call."""

    call_id: Required[str]

    output: Required[Iterable[InputUnionMember1ShellCallOutputOutput]]

    type: Required[Literal["shell_call_output"]]

    id: Optional[str]

    max_output_length: Optional[int]


class InputUnionMember1ApplyPatchCallOperationApplyPatchCallOperationCreateFile(  # type: ignore[call-arg]
    TypedDict, total=False, extra_items=object
):
    """Instruction for creating a new file via the apply_patch tool."""

    diff: Required[str]

    path: Required[str]

    type: Required[Literal["create_file"]]


class InputUnionMember1ApplyPatchCallOperationApplyPatchCallOperationDeleteFile(  # type: ignore[call-arg]
    TypedDict, total=False, extra_items=object
):
    """Instruction for deleting an existing file via the apply_patch tool."""

    path: Required[str]

    type: Required[Literal["delete_file"]]


class InputUnionMember1ApplyPatchCallOperationApplyPatchCallOperationUpdateFile(  # type: ignore[call-arg]
    TypedDict, total=False, extra_items=object
):
    """Instruction for updating an existing file via the apply_patch tool."""

    diff: Required[str]

    path: Required[str]

    type: Required[Literal["update_file"]]


InputUnionMember1ApplyPatchCallOperation: TypeAlias = Union[
    InputUnionMember1ApplyPatchCallOperationApplyPatchCallOperationCreateFile,
    InputUnionMember1ApplyPatchCallOperationApplyPatchCallOperationDeleteFile,
    InputUnionMember1ApplyPatchCallOperationApplyPatchCallOperationUpdateFile,
]


class InputUnionMember1ApplyPatchCall(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """
    A tool call representing a request to create, delete, or update files using diff patches.
    """

    call_id: Required[str]

    operation: Required[InputUnionMember1ApplyPatchCallOperation]
    """Instruction for creating a new file via the apply_patch tool."""

    status: Required[Literal["in_progress", "completed"]]

    type: Required[Literal["apply_patch_call"]]

    id: Optional[str]


class InputUnionMember1ApplyPatchCallOutput(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """The streamed output emitted by an apply patch tool call."""

    call_id: Required[str]

    status: Required[Literal["completed", "failed"]]

    type: Required[Literal["apply_patch_call_output"]]

    id: Optional[str]

    output: Optional[str]


class InputUnionMember1McpListToolsTool(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A tool available on an MCP server."""

    input_schema: Required[object]

    name: Required[str]

    annotations: object

    description: Optional[str]


class InputUnionMember1McpListTools(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A list of tools available on an MCP server."""

    id: Required[str]

    server_label: Required[str]

    tools: Required[Iterable[InputUnionMember1McpListToolsTool]]

    type: Required[Literal["mcp_list_tools"]]

    error: Optional[str]


class InputUnionMember1McpApprovalResponse(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A response to an MCP approval request."""

    approval_request_id: Required[str]

    approve: Required[bool]

    type: Required[Literal["mcp_approval_response"]]

    id: Optional[str]

    reason: Optional[str]


class InputUnionMember1McpCall(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """An invocation of a tool on an MCP server."""

    id: Required[str]

    arguments: Required[str]

    name: Required[str]

    server_label: Required[str]

    type: Required[Literal["mcp_call"]]

    approval_request_id: Optional[str]

    error: Optional[str]

    output: Optional[str]

    status: Literal["in_progress", "completed", "incomplete", "calling", "failed"]


InputUnionMember1ResponseCustomToolCallOutputParamOutputUnionMember1: TypeAlias = Union[
    ResponseInputTextParam, ResponseInputImageParam, ResponseInputFileParam
]


class InputUnionMember1ResponseCustomToolCallOutputParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """The output of a custom tool call from your code, being sent back to the model."""

    call_id: Required[str]

    output: Required[Union[str, Iterable[InputUnionMember1ResponseCustomToolCallOutputParamOutputUnionMember1]]]

    type: Required[Literal["custom_tool_call_output"]]

    id: str


class InputUnionMember1ResponseCustomToolCallParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A call to a custom tool created by the model."""

    call_id: Required[str]

    input: Required[str]

    name: Required[str]

    type: Required[Literal["custom_tool_call"]]

    id: str


class InputUnionMember1ItemReference(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """An internal identifier for an item to reference."""

    id: Required[str]

    type: Optional[Literal["item_reference"]]


InputUnionMember1ResponseOutputMessageContentResponseOutputTextAnnotation: TypeAlias = Union[
    AnnotationFileCitationParam,
    AnnotationURLCitationParam,
    AnnotationContainerFileCitationParam,
    AnnotationFilePathParam,
]


class InputUnionMember1ResponseOutputMessageContentResponseOutputText(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A text output from the model."""

    annotations: Required[Iterable[InputUnionMember1ResponseOutputMessageContentResponseOutputTextAnnotation]]

    text: Required[str]

    type: Required[Literal["output_text"]]

    logprobs: Optional[Iterable[LogprobParam]]


class InputUnionMember1ResponseOutputMessageContentResponseOutputRefusal(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A refusal from the model."""

    refusal: Required[str]

    type: Required[Literal["refusal"]]


InputUnionMember1ResponseOutputMessageContent: TypeAlias = Union[
    InputUnionMember1ResponseOutputMessageContentResponseOutputText,
    InputUnionMember1ResponseOutputMessageContentResponseOutputRefusal,
]


class InputUnionMember1ResponseOutputMessage(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """An output message from the model."""

    id: Required[str]

    content: Required[Iterable[InputUnionMember1ResponseOutputMessageContent]]

    role: Required[Literal["assistant"]]

    status: Required[Literal["in_progress", "completed", "incomplete"]]

    type: Required[Literal["message"]]


class InputUnionMember1ResponseFileSearchToolCallResult(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    attributes: Optional[Dict[str, Union[str, float, bool]]]

    file_id: Optional[str]

    filename: Optional[str]

    score: Optional[float]

    text: Optional[str]


class InputUnionMember1ResponseFileSearchToolCall(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """The results of a file search tool call.

    See the
    [file search guide](https://platform.openai.com/docs/guides/tools-file-search) for more information.
    """

    id: Required[str]

    queries: Required[SequenceNotStr[str]]

    status: Required[Literal["in_progress", "searching", "completed", "incomplete", "failed"]]

    type: Required[Literal["file_search_call"]]

    results: Optional[Iterable[InputUnionMember1ResponseFileSearchToolCallResult]]


class InputUnionMember1ResponseFunctionToolCall(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A tool call to run a function.

    See the
    [function calling guide](https://platform.openai.com/docs/guides/function-calling) for more information.
    """

    arguments: Required[str]

    call_id: Required[str]

    name: Required[str]

    type: Required[Literal["function_call"]]

    id: Optional[str]

    status: Optional[Literal["in_progress", "completed", "incomplete"]]


class InputUnionMember1ResponseFunctionWebSearchActionActionSearch(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """Action type "search" - Performs a web search query."""

    query: Required[str]

    type: Required[Literal["search"]]

    queries: Optional[SequenceNotStr[str]]

    sources: Optional[Iterable[ActionSearchSourceParam]]


InputUnionMember1ResponseFunctionWebSearchAction: TypeAlias = Union[
    InputUnionMember1ResponseFunctionWebSearchActionActionSearch, ActionOpenPageParam, ActionFindParam
]


class InputUnionMember1ResponseFunctionWebSearch(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """The results of a web search tool call.

    See the
    [web search guide](https://platform.openai.com/docs/guides/tools-web-search) for more information.
    """

    id: Required[str]

    action: Required[InputUnionMember1ResponseFunctionWebSearchAction]
    """Action type "search" - Performs a web search query."""

    status: Required[Literal["in_progress", "searching", "completed", "failed"]]

    type: Required[Literal["web_search_call"]]


InputUnionMember1ResponseComputerToolCallAction: TypeAlias = Union[
    ActionClickParam,
    ActionDoubleClickParam,
    ActionDragParam,
    ActionKeypressParam,
    ActionMoveParam,
    ActionScreenshotParam,
    ActionScrollParam,
    ActionTypeParam,
    ActionWaitParam,
]


class InputUnionMember1ResponseComputerToolCallPendingSafetyCheck(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A pending safety check for the computer call."""

    id: Required[str]

    code: Optional[str]

    message: Optional[str]


class InputUnionMember1ResponseComputerToolCall(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A tool call to a computer use tool.

    See the
    [computer use guide](https://platform.openai.com/docs/guides/tools-computer-use) for more information.
    """

    id: Required[str]

    action: Required[InputUnionMember1ResponseComputerToolCallAction]
    """A click action."""

    call_id: Required[str]

    pending_safety_checks: Required[Iterable[InputUnionMember1ResponseComputerToolCallPendingSafetyCheck]]

    status: Required[Literal["in_progress", "completed", "incomplete"]]

    type: Required[Literal["computer_call"]]


class InputUnionMember1ResponseReasoningItemContent(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """Reasoning text from the model."""

    text: Required[str]

    type: Required[Literal["reasoning_text"]]


class InputUnionMember1ResponseReasoningItem(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """
    A description of the chain of thought used by a reasoning model while generating
    a response. Be sure to include these items in your `input` to the Responses API
    for subsequent turns of a conversation if you are manually
    [managing context](https://platform.openai.com/docs/guides/conversation-state).
    """

    id: Required[str]

    summary: Required[Iterable[SummaryParam]]

    type: Required[Literal["reasoning"]]

    content: Optional[Iterable[InputUnionMember1ResponseReasoningItemContent]]

    encrypted_content: Optional[str]

    status: Optional[Literal["in_progress", "completed", "incomplete"]]


class InputUnionMember1ResponseCompactionItem(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """
    A compaction item generated by the [`v1/responses/compact` API](https://platform.openai.com/docs/api-reference/responses/compact).
    """

    id: Required[str]

    encrypted_content: Required[str]

    type: Required[Literal["compaction"]]

    created_by: Optional[str]


InputUnionMember1ResponseCodeInterpreterToolCallOutput: TypeAlias = Union[OutputLogsParam, OutputImageParam]


class InputUnionMember1ResponseCodeInterpreterToolCall(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A tool call to run code."""

    id: Required[str]

    container_id: Required[str]

    status: Required[Literal["in_progress", "completed", "incomplete", "interpreting", "failed"]]

    type: Required[Literal["code_interpreter_call"]]

    code: Optional[str]

    outputs: Optional[Iterable[InputUnionMember1ResponseCodeInterpreterToolCallOutput]]


class InputUnionMember1ResponseFunctionShellToolCallAction(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """The shell commands and limits that describe how to run the tool call."""

    commands: Required[SequenceNotStr[str]]

    max_output_length: Optional[int]

    timeout_ms: Optional[int]


class InputUnionMember1ResponseFunctionShellToolCall(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A tool call that executes one or more shell commands in a managed environment."""

    id: Required[str]

    action: Required[InputUnionMember1ResponseFunctionShellToolCallAction]
    """The shell commands and limits that describe how to run the tool call."""

    call_id: Required[str]

    status: Required[Literal["in_progress", "completed", "incomplete"]]

    type: Required[Literal["shell_call"]]

    created_by: Optional[str]


class InputUnionMember1ResponseFunctionShellToolCallOutputOutputOutcomeOutputOutcomeTimeout(  # type: ignore[call-arg]
    TypedDict, total=False, extra_items=object
):
    """Indicates that the shell call exceeded its configured time limit."""

    type: Required[Literal["timeout"]]


class InputUnionMember1ResponseFunctionShellToolCallOutputOutputOutcomeOutputOutcomeExit(  # type: ignore[call-arg]
    TypedDict, total=False, extra_items=object
):
    """Indicates that the shell commands finished and returned an exit code."""

    exit_code: Required[int]

    type: Required[Literal["exit"]]


InputUnionMember1ResponseFunctionShellToolCallOutputOutputOutcome: TypeAlias = Union[
    InputUnionMember1ResponseFunctionShellToolCallOutputOutputOutcomeOutputOutcomeTimeout,
    InputUnionMember1ResponseFunctionShellToolCallOutputOutputOutcomeOutputOutcomeExit,
]


class InputUnionMember1ResponseFunctionShellToolCallOutputOutput(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """The content of a shell tool call output that was emitted."""

    outcome: Required[InputUnionMember1ResponseFunctionShellToolCallOutputOutputOutcome]
    """Indicates that the shell call exceeded its configured time limit."""

    stderr: Required[str]

    stdout: Required[str]

    created_by: Optional[str]


class InputUnionMember1ResponseFunctionShellToolCallOutput(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """The output of a shell tool call that was emitted."""

    id: Required[str]

    call_id: Required[str]

    output: Required[Iterable[InputUnionMember1ResponseFunctionShellToolCallOutputOutput]]

    status: Required[Literal["in_progress", "completed", "incomplete"]]

    type: Required[Literal["shell_call_output"]]

    created_by: Optional[str]

    max_output_length: Optional[int]


class InputUnionMember1ResponseApplyPatchToolCallOperationOperationCreateFile(  # type: ignore[call-arg]
    TypedDict, total=False, extra_items=object
):
    """Instruction describing how to create a file via the apply_patch tool."""

    diff: Required[str]

    path: Required[str]

    type: Required[Literal["create_file"]]


class InputUnionMember1ResponseApplyPatchToolCallOperationOperationDeleteFile(  # type: ignore[call-arg]
    TypedDict, total=False, extra_items=object
):
    """Instruction describing how to delete a file via the apply_patch tool."""

    path: Required[str]

    type: Required[Literal["delete_file"]]


class InputUnionMember1ResponseApplyPatchToolCallOperationOperationUpdateFile(  # type: ignore[call-arg]
    TypedDict, total=False, extra_items=object
):
    """Instruction describing how to update a file via the apply_patch tool."""

    diff: Required[str]

    path: Required[str]

    type: Required[Literal["update_file"]]


InputUnionMember1ResponseApplyPatchToolCallOperation: TypeAlias = Union[
    InputUnionMember1ResponseApplyPatchToolCallOperationOperationCreateFile,
    InputUnionMember1ResponseApplyPatchToolCallOperationOperationDeleteFile,
    InputUnionMember1ResponseApplyPatchToolCallOperationOperationUpdateFile,
]


class InputUnionMember1ResponseApplyPatchToolCall(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A tool call that applies file diffs by creating, deleting, or updating files."""

    id: Required[str]

    call_id: Required[str]

    operation: Required[InputUnionMember1ResponseApplyPatchToolCallOperation]
    """Instruction describing how to create a file via the apply_patch tool."""

    status: Required[Literal["in_progress", "completed"]]

    type: Required[Literal["apply_patch_call"]]

    created_by: Optional[str]


class InputUnionMember1ResponseApplyPatchToolCallOutput(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """The output emitted by an apply patch tool call."""

    id: Required[str]

    call_id: Required[str]

    status: Required[Literal["completed", "failed"]]

    type: Required[Literal["apply_patch_call_output"]]

    created_by: Optional[str]

    output: Optional[str]


class InputUnionMember1ResponseCustomToolCall(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A call to a custom tool created by the model."""

    call_id: Required[str]

    input: Required[str]

    name: Required[str]

    type: Required[Literal["custom_tool_call"]]

    id: Optional[str]


InputUnionMember1: TypeAlias = Union[
    InputUnionMember1EasyInputMessageParam,
    InputUnionMember1Message,
    InputUnionMember1ResponseOutputMessageParam,
    InputUnionMember1ResponseFileSearchToolCallParam,
    InputUnionMember1ResponseComputerToolCallParam,
    InputUnionMember1ComputerCallOutput,
    InputUnionMember1ResponseFunctionWebSearchParam,
    InputUnionMember1ResponseFunctionToolCallParam,
    InputUnionMember1FunctionCallOutput,
    InputUnionMember1ResponseReasoningItemParam,
    InputUnionMember1ResponseCompactionItemParamParam,
    InputUnionMember1ImageGenerationCall,
    InputUnionMember1ResponseCodeInterpreterToolCallParam,
    InputUnionMember1LocalShellCall,
    InputUnionMember1LocalShellCallOutput,
    InputUnionMember1ShellCall,
    InputUnionMember1ShellCallOutput,
    InputUnionMember1ApplyPatchCall,
    InputUnionMember1ApplyPatchCallOutput,
    InputUnionMember1McpListTools,
    McpApprovalRequestParam,
    InputUnionMember1McpApprovalResponse,
    InputUnionMember1McpCall,
    InputUnionMember1ResponseCustomToolCallOutputParam,
    InputUnionMember1ResponseCustomToolCallParam,
    InputUnionMember1ItemReference,
    InputUnionMember1ResponseOutputMessage,
    InputUnionMember1ResponseFileSearchToolCall,
    InputUnionMember1ResponseFunctionToolCall,
    InputUnionMember1ResponseFunctionWebSearch,
    InputUnionMember1ResponseComputerToolCall,
    InputUnionMember1ResponseReasoningItem,
    InputUnionMember1ResponseCompactionItem,
    InputUnionMember1ImageGenerationCall,
    InputUnionMember1ResponseCodeInterpreterToolCall,
    InputUnionMember1LocalShellCall,
    InputUnionMember1ResponseFunctionShellToolCall,
    InputUnionMember1ResponseFunctionShellToolCallOutput,
    InputUnionMember1ResponseApplyPatchToolCall,
    InputUnionMember1ResponseApplyPatchToolCallOutput,
    InputUnionMember1McpCall,
    InputUnionMember1McpListTools,
    McpApprovalRequestParam,
    InputUnionMember1ResponseCustomToolCall,
]

PreviousInputMessage: TypeAlias = Union[MessageParam, Dict[str, object]]


class PromptVariablesResponseInputText(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A text input to the model."""

    text: Required[str]

    type: Required[Literal["input_text"]]


class PromptVariablesResponseInputImage(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """An image input to the model.

    Learn about [image inputs](https://platform.openai.com/docs/guides/vision).
    """

    detail: Required[Literal["low", "high", "auto"]]

    type: Required[Literal["input_image"]]

    file_id: Optional[str]

    image_url: Optional[str]


class PromptVariablesResponseInputFile(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A file input to the model."""

    type: Required[Literal["input_file"]]

    file_data: Optional[str]

    file_id: Optional[str]

    file_url: Optional[str]

    filename: Optional[str]


PromptVariables: TypeAlias = Union[
    str, PromptVariablesResponseInputText, PromptVariablesResponseInputImage, PromptVariablesResponseInputFile
]


class Prompt(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """
    Reference to a prompt template and its variables.
    [Learn more](https://platform.openai.com/docs/guides/text?api-mode=responses#reusable-prompts).
    """

    id: Required[str]

    variables: Optional[Dict[str, PromptVariables]]

    version: Optional[str]


class Reasoning(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """**gpt-5 and o-series models only**

    Configuration options for
    [reasoning models](https://platform.openai.com/docs/guides/reasoning).
    """

    effort: Optional[Literal["none", "minimal", "low", "medium", "high", "xhigh"]]

    generate_summary: Optional[Literal["auto", "concise", "detailed"]]

    summary: Optional[Literal["auto", "concise", "detailed"]]


class TextFormatResponseFormatText(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """Default response format. Used to generate text responses."""

    type: Required[Literal["text"]]


class TextFormatResponseFormatTextJsonSchemaConfig(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """JSON Schema response format.

    Used to generate structured JSON responses.
    Learn more about [Structured Outputs](https://platform.openai.com/docs/guides/structured-outputs).
    """

    name: Required[str]

    schema: Required[Dict[str, object]]

    type: Required[Literal["json_schema"]]

    description: Optional[str]

    strict: Optional[bool]


class TextFormatResponseFormatJsonObject(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """JSON object response format.

    An older method of generating JSON responses.
    Using `json_schema` is recommended for models that support it. Note that the
    model will not generate JSON without a system or user message instructing it
    to do so.
    """

    type: Required[Literal["json_object"]]


TextFormat: TypeAlias = Union[
    TextFormatResponseFormatText, TextFormatResponseFormatTextJsonSchemaConfig, TextFormatResponseFormatJsonObject
]


class Text(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """Configuration options for a text response from the model.

    Can be plain
    text or structured JSON data. Learn more:
    - [Text inputs and outputs](https://platform.openai.com/docs/guides/text)
    - [Structured Outputs](https://platform.openai.com/docs/guides/structured-outputs)
    """

    format: Optional[TextFormat]
    """Default response format. Used to generate text responses."""

    verbosity: Optional[Literal["low", "medium", "high"]]


class ToolChoiceToolChoiceAllowed(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """Constrains the tools available to the model to a pre-defined set."""

    mode: Required[Literal["auto", "required"]]

    tools: Required[Iterable[Dict[str, object]]]

    type: Required[Literal["allowed_tools"]]


class ToolChoiceToolChoiceTypes(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """
    Indicates that the model should use a built-in tool to generate a response.
    [Learn more about built-in tools](https://platform.openai.com/docs/guides/tools).
    """

    type: Required[
        Literal[
            "file_search",
            "web_search_preview",
            "computer_use_preview",
            "web_search_preview_2025_03_11",
            "image_generation",
            "code_interpreter",
        ]
    ]


class ToolChoiceToolChoiceFunction(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """Use this option to force the model to call a specific function."""

    name: Required[str]

    type: Required[Literal["function"]]


class ToolChoiceToolChoiceMcp(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """
    Use this option to force the model to call a specific tool on a remote MCP server.
    """

    server_label: Required[str]

    type: Required[Literal["mcp"]]

    name: Optional[str]


class ToolChoiceToolChoiceCustom(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """Use this option to force the model to call a specific custom tool."""

    name: Required[str]

    type: Required[Literal["custom"]]


class ToolChoiceToolChoiceApplyPatch(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """Forces the model to call the apply_patch tool when executing a tool call."""

    type: Required[Literal["apply_patch"]]


class ToolChoiceToolChoiceShell(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """Forces the model to call the shell tool when a tool call is required."""

    type: Required[Literal["shell"]]


ToolChoice: TypeAlias = Union[
    Literal["none", "auto", "required"],
    ToolChoiceToolChoiceAllowed,
    ToolChoiceToolChoiceTypes,
    ToolChoiceToolChoiceFunction,
    ToolChoiceToolChoiceMcp,
    ToolChoiceToolChoiceCustom,
    ToolChoiceToolChoiceApplyPatch,
    ToolChoiceToolChoiceShell,
]


class ToolFunctionTool(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """Defines a function in your own code the model can choose to call.

    Learn more about [function calling](https://platform.openai.com/docs/guides/function-calling).
    """

    name: Required[str]

    type: Required[Literal["function"]]

    description: Optional[str]

    parameters: Optional[Dict[str, object]]

    strict: Optional[bool]


ToolFileSearchToolFiltersCompoundFilterFilter: TypeAlias = Union[ComparisonFilterParam, object]


class ToolFileSearchToolFiltersCompoundFilter(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """Combine multiple filters using `and` or `or`."""

    filters: Required[Iterable[ToolFileSearchToolFiltersCompoundFilterFilter]]

    type: Required[Literal["and", "or"]]


ToolFileSearchToolFilters: TypeAlias = Union[ComparisonFilterParam, ToolFileSearchToolFiltersCompoundFilter]


class ToolFileSearchToolRankingOptionsHybridSearch(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """
    Weights that control how reciprocal rank fusion balances semantic embedding matches versus sparse keyword matches when hybrid search is enabled.
    """

    embedding_weight: Required[float]

    text_weight: Required[float]


class ToolFileSearchToolRankingOptions(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """Ranking options for search."""

    hybrid_search: Optional[ToolFileSearchToolRankingOptionsHybridSearch]
    """
    Weights that control how reciprocal rank fusion balances semantic embedding
    matches versus sparse keyword matches when hybrid search is enabled.
    """

    ranker: Optional[Literal["auto", "default-2024-11-15"]]

    score_threshold: Optional[float]


class ToolFileSearchTool(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A tool that searches for relevant content from uploaded files.

    Learn more about the [file search tool](https://platform.openai.com/docs/guides/tools-file-search).
    """

    type: Required[Literal["file_search"]]

    vector_store_ids: Required[SequenceNotStr[str]]

    filters: Optional[ToolFileSearchToolFilters]
    """
    A filter used to compare a specified attribute key to a given value using a
    defined comparison operation.
    """

    max_num_results: Optional[int]

    ranking_options: Optional[ToolFileSearchToolRankingOptions]
    """Ranking options for search."""


class ToolComputerTool(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A tool that controls a virtual computer.

    Learn more about the [computer tool](https://platform.openai.com/docs/guides/tools-computer-use).
    """

    display_height: Required[int]

    display_width: Required[int]

    environment: Required[Literal["windows", "mac", "linux", "ubuntu", "browser"]]

    type: Required[Literal["computer_use_preview"]]


class ToolWebSearchToolFilters(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """Filters for the search."""

    allowed_domains: Optional[SequenceNotStr[str]]


class ToolWebSearchToolUserLocation(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """The approximate location of the user."""

    city: Optional[str]

    country: Optional[str]

    region: Optional[str]

    timezone: Optional[str]

    type: Optional[Literal["approximate"]]


class ToolWebSearchTool(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """Search the Internet for sources related to the prompt.

    Learn more about the
    [web search tool](https://platform.openai.com/docs/guides/tools-web-search).
    """

    type: Required[Literal["web_search", "web_search_2025_08_26"]]

    filters: Optional[ToolWebSearchToolFilters]
    """Filters for the search."""

    search_context_size: Optional[Literal["low", "medium", "high"]]

    user_location: Optional[ToolWebSearchToolUserLocation]
    """The approximate location of the user."""


class ToolMcpAllowedToolsMcpAllowedToolsMcpToolFilter(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A filter object to specify which tools are allowed."""

    read_only: Optional[bool]

    tool_names: Optional[SequenceNotStr[str]]


ToolMcpAllowedTools: TypeAlias = Union[SequenceNotStr[str], ToolMcpAllowedToolsMcpAllowedToolsMcpToolFilter]


class ToolMcpRequireApprovalMcpRequireApprovalMcpToolApprovalFilterAlways(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A filter object to specify which tools are allowed."""

    read_only: Optional[bool]

    tool_names: Optional[SequenceNotStr[str]]


class ToolMcpRequireApprovalMcpRequireApprovalMcpToolApprovalFilterNever(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A filter object to specify which tools are allowed."""

    read_only: Optional[bool]

    tool_names: Optional[SequenceNotStr[str]]


class ToolMcpRequireApprovalMcpRequireApprovalMcpToolApprovalFilter(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """Specify which of the MCP server's tools require approval.

    Can be
    `always`, `never`, or a filter object associated with tools
    that require approval.
    """

    always: Optional[ToolMcpRequireApprovalMcpRequireApprovalMcpToolApprovalFilterAlways]
    """A filter object to specify which tools are allowed."""

    never: Optional[ToolMcpRequireApprovalMcpRequireApprovalMcpToolApprovalFilterNever]
    """A filter object to specify which tools are allowed."""


ToolMcpRequireApproval: TypeAlias = Union[
    ToolMcpRequireApprovalMcpRequireApprovalMcpToolApprovalFilter, Literal["always", "never"]
]


class ToolMcp(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """
    Give the model access to additional tools via remote Model Context Protocol
    (MCP) servers. [Learn more about MCP](https://platform.openai.com/docs/guides/tools-remote-mcp).
    """

    server_label: Required[str]

    type: Required[Literal["mcp"]]

    allowed_tools: Optional[ToolMcpAllowedTools]
    """A filter object to specify which tools are allowed."""

    authorization: Optional[str]

    connector_id: Optional[
        Literal[
            "connector_dropbox",
            "connector_gmail",
            "connector_googlecalendar",
            "connector_googledrive",
            "connector_microsoftteams",
            "connector_outlookcalendar",
            "connector_outlookemail",
            "connector_sharepoint",
        ]
    ]

    headers: Optional[Dict[str, str]]

    require_approval: Optional[ToolMcpRequireApproval]
    """Specify which of the MCP server's tools require approval.

    Can be `always`, `never`, or a filter object associated with tools that require
    approval.
    """

    server_description: Optional[str]

    server_url: Optional[str]


class ToolCodeInterpreterContainerCodeInterpreterContainerCodeInterpreterToolAuto(  # type: ignore[call-arg]
    TypedDict, total=False, extra_items=object
):
    """Configuration for a code interpreter container.

    Optionally specify the IDs of the files to run the code on.
    """

    type: Required[Literal["auto"]]

    file_ids: Optional[SequenceNotStr[str]]

    memory_limit: Optional[Literal["1g", "4g", "16g", "64g"]]


ToolCodeInterpreterContainer: TypeAlias = Union[
    str, ToolCodeInterpreterContainerCodeInterpreterContainerCodeInterpreterToolAuto
]


class ToolCodeInterpreter(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A tool that runs Python code to help generate a response to a prompt."""

    container: Required[ToolCodeInterpreterContainer]
    """Configuration for a code interpreter container.

    Optionally specify the IDs of the files to run the code on.
    """

    type: Required[Literal["code_interpreter"]]


class ToolImageGenerationInputImageMask(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """Optional mask for inpainting.

    Contains `image_url`
    (string, optional) and `file_id` (string, optional).
    """

    file_id: Optional[str]

    image_url: Optional[str]


class ToolImageGeneration(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A tool that generates images using the GPT image models."""

    type: Required[Literal["image_generation"]]

    background: Optional[Literal["transparent", "opaque", "auto"]]

    input_fidelity: Optional[Literal["high", "low"]]

    input_image_mask: Optional[ToolImageGenerationInputImageMask]
    """Optional mask for inpainting.

    Contains `image_url` (string, optional) and `file_id` (string, optional).
    """

    model: Union[str, Literal["gpt-image-1", "gpt-image-1-mini"], None]

    moderation: Optional[Literal["auto", "low"]]

    output_compression: Optional[int]

    output_format: Optional[Literal["png", "webp", "jpeg"]]

    partial_images: Optional[int]

    quality: Optional[Literal["low", "medium", "high", "auto"]]

    size: Optional[Literal["1024x1024", "1024x1536", "1536x1024", "auto"]]


class ToolLocalShell(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A tool that allows the model to execute shell commands in a local environment."""

    type: Required[Literal["local_shell"]]


class ToolFunctionShellTool(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A tool that allows the model to execute shell commands."""

    type: Required[Literal["shell"]]


class ToolCustomToolFormatText(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """Unconstrained free-form text."""

    type: Required[Literal["text"]]


class ToolCustomToolFormatGrammar(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A grammar defined by the user."""

    definition: Required[str]

    syntax: Required[Literal["lark", "regex"]]

    type: Required[Literal["grammar"]]


ToolCustomToolFormat: TypeAlias = Union[ToolCustomToolFormatText, ToolCustomToolFormatGrammar]


class ToolCustomTool(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A custom tool that processes input using a specified format.

    Learn more about   [custom tools](https://platform.openai.com/docs/guides/function-calling#custom-tools)
    """

    name: Required[str]

    type: Required[Literal["custom"]]

    description: Optional[str]

    format: Optional[ToolCustomToolFormat]
    """Unconstrained free-form text."""


class ToolWebSearchPreviewToolUserLocation(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """The user's location."""

    type: Required[Literal["approximate"]]

    city: Optional[str]

    country: Optional[str]

    region: Optional[str]

    timezone: Optional[str]


class ToolWebSearchPreviewTool(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """This tool searches the web for relevant results to use in a response.

    Learn more about the [web search tool](https://platform.openai.com/docs/guides/tools-web-search).
    """

    type: Required[Literal["web_search_preview", "web_search_preview_2025_03_11"]]

    search_context_size: Optional[Literal["low", "medium", "high"]]

    user_location: Optional[ToolWebSearchPreviewToolUserLocation]
    """The user's location."""


class ToolApplyPatchTool(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """Allows the assistant to create, delete, or update files using unified diffs."""

    type: Required[Literal["apply_patch"]]


Tool: TypeAlias = Union[
    ToolFunctionTool,
    ToolFileSearchTool,
    ToolComputerTool,
    ToolWebSearchTool,
    ToolMcp,
    ToolCodeInterpreter,
    ToolImageGeneration,
    ToolLocalShell,
    ToolFunctionShellTool,
    ToolCustomTool,
    ToolWebSearchPreviewTool,
    ToolApplyPatchTool,
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

__all__ = ["LogprobParam", "TopLogprob"]


class TopLogprob(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """The top log probability of a token."""

    token: Required[str]

    bytes: Required[Iterable[int]]

    logprob: Required[float]


class LogprobParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """The log probability of a token."""

    token: Required[str]

    bytes: Required[Iterable[int]]

    logprob: Required[float]

    top_logprobs: Required[Iterable[TopLogprob]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from .._types import SequenceNotStr

__all__ = ["ActionKeypressParam"]


class ActionKeypressParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A collection of keypresses the model would like to perform."""

    keys: Required[SequenceNotStr[str]]

    type: Required[Literal["keypress"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

from .computer_action_param import ComputerActionParam

__all__ = ["ComputerExecuteParams"]


class ComputerExecuteParams(TypedDict, total=False):
    action: ComputerActionParam



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["ActionClickParam"]


class ActionClickParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A click action."""

    button: Required[Literal["left", "right", "wheel", "back", "forward"]]

    type: Required[Literal["click"]]

    x: Required[int]

    y: Required[int]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["FileParam", "File"]


class File(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    file_data: str

    file_id: str

    filename: str


class FileParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """
    Learn about [file inputs](https://platform.openai.com/docs/guides/text) for text generation.
    """

    file: Required[File]

    type: Required[Literal["file"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["AnnotationFileCitationParam"]


class AnnotationFileCitationParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A citation to a file."""

    file_id: Required[str]

    filename: Required[str]

    index: Required[int]

    type: Required[Literal["file_citation"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import TypedDict

from .computer_action_param import ComputerActionParam

__all__ = ["ComputerBatchParams"]


class ComputerBatchParams(TypedDict, total=False):
    actions: Iterable[ComputerActionParam]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["ActionDoubleClickParam"]


class ActionDoubleClickParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A double click action."""

    type: Required[Literal["double_click"]]

    x: Required[int]

    y: Required[int]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["ResponseInputFileParam"]


class ResponseInputFileParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A file input to the model."""

    type: Required[Literal["input_file"]]

    file_data: str

    file_id: Optional[str]

    file_url: str

    filename: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, TypedDict

__all__ = ["ActionDragParam", "Path"]


class Path(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """An x/y coordinate pair, e.g. `{ x: 100, y: 200 }`."""

    x: Required[int]

    y: Required[int]


class ActionDragParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A drag action."""

    path: Required[Iterable[Path]]

    type: Required[Literal["drag"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["ActionOpenPageParam"]


class ActionOpenPageParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """Action type "open_page" - Opens a specific URL from search results."""

    type: Required[Literal["open_page"]]

    url: Required[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable, Optional
from typing_extensions import Literal

import httpx

from ..types import chat_create_completion_params
from .._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options

__all__ = ["ChatResource", "AsyncChatResource"]


class ChatResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ChatResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/tzafon/lightcone-python#accessing-raw-response-data-eg-headers
        """
        return ChatResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ChatResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/tzafon/lightcone-python#with_streaming_response
        """
        return ChatResourceWithStreamingResponse(self)

    def create_completion(
        self,
        *,
        messages: Iterable[chat_create_completion_params.Message],
        add_generation_prompt: bool | Omit = omit,
        add_special_tokens: bool | Omit = omit,
        allowed_token_ids: Optional[Iterable[int]] | Omit = omit,
        bad_words: SequenceNotStr[str] | Omit = omit,
        cache_salt: Optional[str] | Omit = omit,
        chat_template: Optional[str] | Omit = omit,
        chat_template_kwargs: Optional[Dict[str, object]] | Omit = omit,
        continue_final_message: bool | Omit = omit,
        documents: Optional[Iterable[Dict[str, str]]] | Omit = omit,
        echo: bool | Omit = omit,
        frequency_penalty: Optional[float] | Omit = omit,
        ignore_eos: bool | Omit = omit,
        include_reasoning: bool | Omit = omit,
        include_stop_str_in_output: bool | Omit = omit,
        kv_transfer_params: Optional[Dict[str, object]] | Omit = omit,
        length_penalty: float | Omit = omit,
        logit_bias: Optional[Dict[str, float]] | Omit = omit,
        logits_processors: Optional[SequenceNotStr[chat_create_completion_params.LogitsProcessor]] | Omit = omit,
        logprobs: Optional[bool] | Omit = omit,
        max_completion_tokens: Optional[int] | Omit = omit,
        max_tokens: Optional[int] | Omit = omit,
        min_p: Optional[float] | Omit = omit,
        min_tokens: int | Omit = omit,
        mm_processor_kwargs: Optional[Dict[str, object]] | Omit = omit,
        model: Optional[str] | Omit = omit,
        n: Optional[int] | Omit = omit,
        parallel_tool_calls: Optional[bool] | Omit = omit,
        presence_penalty: Optional[float] | Omit = omit,
        priority: int | Omit = omit,
        prompt_logprobs: Optional[int] | Omit = omit,
        reasoning_effort: Optional[Literal["low", "medium", "high"]] | Omit = omit,
        repetition_penalty: Optional[float] | Omit = omit,
        request_id: str | Omit = omit,
        response_format: Optional[chat_create_completion_params.ResponseFormat] | Omit = omit,
        return_token_ids: Optional[bool] | Omit = omit,
        return_tokens_as_token_ids: Optional[bool] | Omit = omit,
        seed: Optional[int] | Omit = omit,
        skip_special_tokens: bool | Omit = omit,
        spaces_between_special_tokens: bool | Omit = omit,
        stop: Union[str, SequenceNotStr[str], None] | Omit = omit,
        stop_token_ids: Optional[Iterable[int]] | Omit = omit,
        stream: Optional[bool] | Omit = omit,
        stream_options: Optional[chat_create_completion_params.StreamOptions] | Omit = omit,
        structured_outputs: Optional[chat_create_completion_params.StructuredOutputs] | Omit = omit,
        temperature: Optional[float] | Omit = omit,
        tool_choice: Optional[chat_create_completion_params.ToolChoice] | Omit = omit,
        tools: Optional[Iterable[chat_create_completion_params.Tool]] | Omit = omit,
        top_k: Optional[int] | Omit = omit,
        top_logprobs: Optional[int] | Omit = omit,
        top_p: Optional[float] | Omit = omit,
        truncate_prompt_tokens: Optional[int] | Omit = omit,
        use_beam_search: bool | Omit = omit,
        user: Optional[str] | Omit = omit,
        vllm_xargs: Optional[Dict[str, Union[str, float, SequenceNotStr[Union[str, float]]]]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Create Chat Completion

        Args:
          add_generation_prompt: If true, the generation prompt will be added to the chat template. This is a
              parameter used by chat template in tokenizer config of the model.

          add_special_tokens: If true, special tokens (e.g. BOS) will be added to the prompt on top of what is
              added by the chat template. For most models, the chat template takes care of
              adding the special tokens so this should be set to false (as is the default).

          cache_salt: If specified, the prefix cache will be salted with the provided string to
              prevent an attacker to guess prompts in multi-user environments. The salt should
              be random, protected from access by 3rd parties, and long enough to be
              unpredictable (e.g., 43 characters base64-encoded, corresponding to 256 bit).

          chat_template: A Jinja template to use for this conversion. As of transformers v4.44, default
              chat template is no longer allowed, so you must provide a chat template if the
              tokenizer does not define one.

          chat_template_kwargs: Additional keyword args to pass to the template renderer. Will be accessible by
              the chat template.

          continue_final_message: If this is set, the chat will be formatted so that the final message in the chat
              is open-ended, without any EOS tokens. The model will continue this message
              rather than starting a new one. This allows you to "prefill" part of the model's
              response for it. Cannot be used at the same time as `add_generation_prompt`.

          documents: A list of dicts representing documents that will be accessible to the model if
              it is performing RAG (retrieval-augmented generation). If the template does not
              support RAG, this argument will have no effect. We recommend that each document
              should be a dict containing "title" and "text" keys.

          echo: If true, the new message will be prepended with the last message if they belong
              to the same role.

          kv_transfer_params: KVTransfer parameters used for disaggregated serving.

          logits_processors: A list of either qualified names of logits processors, or constructor objects,
              to apply when sampling. A constructor is a JSON object with a required
              'qualname' field specifying the qualified name of the processor class/factory,
              and optional 'args' and 'kwargs' fields containing positional and keyword
              arguments. For example: {'qualname': 'my_module.MyLogitsProcessor', 'args': [1,
              2], 'kwargs': {'param': 'value'}}.

          mm_processor_kwargs: Additional kwargs to pass to the HF processor.

          priority: The priority of the request (lower means earlier handling; default: 0). Any
              priority other than 0 will raise an error if the served model does not use
              priority scheduling.

          request_id: The request_id related to this request. If the caller does not set it, a
              random_uuid will be generated. This id is used through out the inference process
              and return in response.

          return_token_ids: If specified, the result will include token IDs alongside the generated text. In
              streaming mode, prompt_token_ids is included only in the first chunk, and
              token_ids contains the delta tokens for each chunk. This is useful for debugging
              or when you need to map generated text back to input tokens.

          return_tokens_as_token_ids: If specified with 'logprobs', tokens are represented as strings of the form
              'token_id:{token_id}' so that tokens that are not JSON-encodable can be
              identified.

          structured_outputs: Additional kwargs for structured outputs

          vllm_xargs: Additional request parameters with (list of) string or numeric values, used by
              custom extensions.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/chat/completions",
            body=maybe_transform(
                {
                    "messages": messages,
                    "add_generation_prompt": add_generation_prompt,
                    "add_special_tokens": add_special_tokens,
                    "allowed_token_ids": allowed_token_ids,
                    "bad_words": bad_words,
                    "cache_salt": cache_salt,
                    "chat_template": chat_template,
                    "chat_template_kwargs": chat_template_kwargs,
                    "continue_final_message": continue_final_message,
                    "documents": documents,
                    "echo": echo,
                    "frequency_penalty": frequency_penalty,
                    "ignore_eos": ignore_eos,
                    "include_reasoning": include_reasoning,
                    "include_stop_str_in_output": include_stop_str_in_output,
                    "kv_transfer_params": kv_transfer_params,
                    "length_penalty": length_penalty,
                    "logit_bias": logit_bias,
                    "logits_processors": logits_processors,
                    "logprobs": logprobs,
                    "max_completion_tokens": max_completion_tokens,
                    "max_tokens": max_tokens,
                    "min_p": min_p,
                    "min_tokens": min_tokens,
                    "mm_processor_kwargs": mm_processor_kwargs,
                    "model": model,
                    "n": n,
                    "parallel_tool_calls": parallel_tool_calls,
                    "presence_penalty": presence_penalty,
                    "priority": priority,
                    "prompt_logprobs": prompt_logprobs,
                    "reasoning_effort": reasoning_effort,
                    "repetition_penalty": repetition_penalty,
                    "request_id": request_id,
                    "response_format": response_format,
                    "return_token_ids": return_token_ids,
                    "return_tokens_as_token_ids": return_tokens_as_token_ids,
                    "seed": seed,
                    "skip_special_tokens": skip_special_tokens,
                    "spaces_between_special_tokens": spaces_between_special_tokens,
                    "stop": stop,
                    "stop_token_ids": stop_token_ids,
                    "stream": stream,
                    "stream_options": stream_options,
                    "structured_outputs": structured_outputs,
                    "temperature": temperature,
                    "tool_choice": tool_choice,
                    "tools": tools,
                    "top_k": top_k,
                    "top_logprobs": top_logprobs,
                    "top_p": top_p,
                    "truncate_prompt_tokens": truncate_prompt_tokens,
                    "use_beam_search": use_beam_search,
                    "user": user,
                    "vllm_xargs": vllm_xargs,
                },
                chat_create_completion_params.ChatCreateCompletionParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AsyncChatResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncChatResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/tzafon/lightcone-python#accessing-raw-response-data-eg-headers
        """
        return AsyncChatResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncChatResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/tzafon/lightcone-python#with_streaming_response
        """
        return AsyncChatResourceWithStreamingResponse(self)

    async def create_completion(
        self,
        *,
        messages: Iterable[chat_create_completion_params.Message],
        add_generation_prompt: bool | Omit = omit,
        add_special_tokens: bool | Omit = omit,
        allowed_token_ids: Optional[Iterable[int]] | Omit = omit,
        bad_words: SequenceNotStr[str] | Omit = omit,
        cache_salt: Optional[str] | Omit = omit,
        chat_template: Optional[str] | Omit = omit,
        chat_template_kwargs: Optional[Dict[str, object]] | Omit = omit,
        continue_final_message: bool | Omit = omit,
        documents: Optional[Iterable[Dict[str, str]]] | Omit = omit,
        echo: bool | Omit = omit,
        frequency_penalty: Optional[float] | Omit = omit,
        ignore_eos: bool | Omit = omit,
        include_reasoning: bool | Omit = omit,
        include_stop_str_in_output: bool | Omit = omit,
        kv_transfer_params: Optional[Dict[str, object]] | Omit = omit,
        length_penalty: float | Omit = omit,
        logit_bias: Optional[Dict[str, float]] | Omit = omit,
        logits_processors: Optional[SequenceNotStr[chat_create_completion_params.LogitsProcessor]] | Omit = omit,
        logprobs: Optional[bool] | Omit = omit,
        max_completion_tokens: Optional[int] | Omit = omit,
        max_tokens: Optional[int] | Omit = omit,
        min_p: Optional[float] | Omit = omit,
        min_tokens: int | Omit = omit,
        mm_processor_kwargs: Optional[Dict[str, object]] | Omit = omit,
        model: Optional[str] | Omit = omit,
        n: Optional[int] | Omit = omit,
        parallel_tool_calls: Optional[bool] | Omit = omit,
        presence_penalty: Optional[float] | Omit = omit,
        priority: int | Omit = omit,
        prompt_logprobs: Optional[int] | Omit = omit,
        reasoning_effort: Optional[Literal["low", "medium", "high"]] | Omit = omit,
        repetition_penalty: Optional[float] | Omit = omit,
        request_id: str | Omit = omit,
        response_format: Optional[chat_create_completion_params.ResponseFormat] | Omit = omit,
        return_token_ids: Optional[bool] | Omit = omit,
        return_tokens_as_token_ids: Optional[bool] | Omit = omit,
        seed: Optional[int] | Omit = omit,
        skip_special_tokens: bool | Omit = omit,
        spaces_between_special_tokens: bool | Omit = omit,
        stop: Union[str, SequenceNotStr[str], None] | Omit = omit,
        stop_token_ids: Optional[Iterable[int]] | Omit = omit,
        stream: Optional[bool] | Omit = omit,
        stream_options: Optional[chat_create_completion_params.StreamOptions] | Omit = omit,
        structured_outputs: Optional[chat_create_completion_params.StructuredOutputs] | Omit = omit,
        temperature: Optional[float] | Omit = omit,
        tool_choice: Optional[chat_create_completion_params.ToolChoice] | Omit = omit,
        tools: Optional[Iterable[chat_create_completion_params.Tool]] | Omit = omit,
        top_k: Optional[int] | Omit = omit,
        top_logprobs: Optional[int] | Omit = omit,
        top_p: Optional[float] | Omit = omit,
        truncate_prompt_tokens: Optional[int] | Omit = omit,
        use_beam_search: bool | Omit = omit,
        user: Optional[str] | Omit = omit,
        vllm_xargs: Optional[Dict[str, Union[str, float, SequenceNotStr[Union[str, float]]]]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Create Chat Completion

        Args:
          add_generation_prompt: If true, the generation prompt will be added to the chat template. This is a
              parameter used by chat template in tokenizer config of the model.

          add_special_tokens: If true, special tokens (e.g. BOS) will be added to the prompt on top of what is
              added by the chat template. For most models, the chat template takes care of
              adding the special tokens so this should be set to false (as is the default).

          cache_salt: If specified, the prefix cache will be salted with the provided string to
              prevent an attacker to guess prompts in multi-user environments. The salt should
              be random, protected from access by 3rd parties, and long enough to be
              unpredictable (e.g., 43 characters base64-encoded, corresponding to 256 bit).

          chat_template: A Jinja template to use for this conversion. As of transformers v4.44, default
              chat template is no longer allowed, so you must provide a chat template if the
              tokenizer does not define one.

          chat_template_kwargs: Additional keyword args to pass to the template renderer. Will be accessible by
              the chat template.

          continue_final_message: If this is set, the chat will be formatted so that the final message in the chat
              is open-ended, without any EOS tokens. The model will continue this message
              rather than starting a new one. This allows you to "prefill" part of the model's
              response for it. Cannot be used at the same time as `add_generation_prompt`.

          documents: A list of dicts representing documents that will be accessible to the model if
              it is performing RAG (retrieval-augmented generation). If the template does not
              support RAG, this argument will have no effect. We recommend that each document
              should be a dict containing "title" and "text" keys.

          echo: If true, the new message will be prepended with the last message if they belong
              to the same role.

          kv_transfer_params: KVTransfer parameters used for disaggregated serving.

          logits_processors: A list of either qualified names of logits processors, or constructor objects,
              to apply when sampling. A constructor is a JSON object with a required
              'qualname' field specifying the qualified name of the processor class/factory,
              and optional 'args' and 'kwargs' fields containing positional and keyword
              arguments. For example: {'qualname': 'my_module.MyLogitsProcessor', 'args': [1,
              2], 'kwargs': {'param': 'value'}}.

          mm_processor_kwargs: Additional kwargs to pass to the HF processor.

          priority: The priority of the request (lower means earlier handling; default: 0). Any
              priority other than 0 will raise an error if the served model does not use
              priority scheduling.

          request_id: The request_id related to this request. If the caller does not set it, a
              random_uuid will be generated. This id is used through out the inference process
              and return in response.

          return_token_ids: If specified, the result will include token IDs alongside the generated text. In
              streaming mode, prompt_token_ids is included only in the first chunk, and
              token_ids contains the delta tokens for each chunk. This is useful for debugging
              or when you need to map generated text back to input tokens.

          return_tokens_as_token_ids: If specified with 'logprobs', tokens are represented as strings of the form
              'token_id:{token_id}' so that tokens that are not JSON-encodable can be
              identified.

          structured_outputs: Additional kwargs for structured outputs

          vllm_xargs: Additional request parameters with (list of) string or numeric values, used by
              custom extensions.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/chat/completions",
            body=await async_maybe_transform(
                {
                    "messages": messages,
                    "add_generation_prompt": add_generation_prompt,
                    "add_special_tokens": add_special_tokens,
                    "allowed_token_ids": allowed_token_ids,
                    "bad_words": bad_words,
                    "cache_salt": cache_salt,
                    "chat_template": chat_template,
                    "chat_template_kwargs": chat_template_kwargs,
                    "continue_final_message": continue_final_message,
                    "documents": documents,
                    "echo": echo,
                    "frequency_penalty": frequency_penalty,
                    "ignore_eos": ignore_eos,
                    "include_reasoning": include_reasoning,
                    "include_stop_str_in_output": include_stop_str_in_output,
                    "kv_transfer_params": kv_transfer_params,
                    "length_penalty": length_penalty,
                    "logit_bias": logit_bias,
                    "logits_processors": logits_processors,
                    "logprobs": logprobs,
                    "max_completion_tokens": max_completion_tokens,
                    "max_tokens": max_tokens,
                    "min_p": min_p,
                    "min_tokens": min_tokens,
                    "mm_processor_kwargs": mm_processor_kwargs,
                    "model": model,
                    "n": n,
                    "parallel_tool_calls": parallel_tool_calls,
                    "presence_penalty": presence_penalty,
                    "priority": priority,
                    "prompt_logprobs": prompt_logprobs,
                    "reasoning_effort": reasoning_effort,
                    "repetition_penalty": repetition_penalty,
                    "request_id": request_id,
                    "response_format": response_format,
                    "return_token_ids": return_token_ids,
                    "return_tokens_as_token_ids": return_tokens_as_token_ids,
                    "seed": seed,
                    "skip_special_tokens": skip_special_tokens,
                    "spaces_between_special_tokens": spaces_between_special_tokens,
                    "stop": stop,
                    "stop_token_ids": stop_token_ids,
                    "stream": stream,
                    "stream_options": stream_options,
                    "structured_outputs": structured_outputs,
                    "temperature": temperature,
                    "tool_choice": tool_choice,
                    "tools": tools,
                    "top_k": top_k,
                    "top_logprobs": top_logprobs,
                    "top_p": top_p,
                    "truncate_prompt_tokens": truncate_prompt_tokens,
                    "use_beam_search": use_beam_search,
                    "user": user,
                    "vllm_xargs": vllm_xargs,
                },
                chat_create_completion_params.ChatCreateCompletionParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class ChatResourceWithRawResponse:
    def __init__(self, chat: ChatResource) -> None:
        self._chat = chat

        self.create_completion = to_raw_response_wrapper(
            chat.create_completion,
        )


class AsyncChatResourceWithRawResponse:
    def __init__(self, chat: AsyncChatResource) -> None:
        self._chat = chat

        self.create_completion = async_to_raw_response_wrapper(
            chat.create_completion,
        )


class ChatResourceWithStreamingResponse:
    def __init__(self, chat: ChatResource) -> None:
        self._chat = chat

        self.create_completion = to_streamed_response_wrapper(
            chat.create_completion,
        )


class AsyncChatResourceWithStreamingResponse:
    def __init__(self, chat: AsyncChatResource) -> None:
        self._chat = chat

        self.create_completion = async_to_streamed_response_wrapper(
            chat.create_completion,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["ResponseInputImageParam"]


class ResponseInputImageParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """An image input to the model.

    Learn about [image inputs](https://platform.openai.com/docs/guides/vision).
    """

    detail: Required[Literal["low", "high", "auto"]]

    type: Required[Literal["input_image"]]

    file_id: Optional[str]

    image_url: Optional[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["ChatCompletionMessageFunctionToolCallParam", "Function"]


class Function(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """The function that the model called."""

    arguments: Required[str]

    name: Required[str]


class ChatCompletionMessageFunctionToolCallParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A call to a function tool created by the model."""

    id: Required[str]

    function: Required[Function]
    """The function that the model called."""

    type: Required[Literal["function"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["AnnotationURLCitationParam"]


class AnnotationURLCitationParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A citation for a web resource used to generate a model response."""

    end_index: Required[int]

    start_index: Required[int]

    title: Required[str]

    type: Required[Literal["url_citation"]]

    url: Required[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tzafon import Lightcone, AsyncLightcone
from tests.utils import assert_matches_type
from tzafon.types.agent import (
    TaskPauseResponse,
    TaskStartResponse,
    TaskResumeResponse,
    TaskInjectMessageResponse,
    TaskRetrieveStatusResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestTasks:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_inject_message(self, client: Lightcone) -> None:
        task = client.agent.tasks.inject_message(
            id="id",
        )
        assert_matches_type(TaskInjectMessageResponse, task, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_inject_message_with_all_params(self, client: Lightcone) -> None:
        task = client.agent.tasks.inject_message(
            id="id",
            message="message",
        )
        assert_matches_type(TaskInjectMessageResponse, task, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_inject_message(self, client: Lightcone) -> None:
        response = client.agent.tasks.with_raw_response.inject_message(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        task = response.parse()
        assert_matches_type(TaskInjectMessageResponse, task, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_inject_message(self, client: Lightcone) -> None:
        with client.agent.tasks.with_streaming_response.inject_message(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            task = response.parse()
            assert_matches_type(TaskInjectMessageResponse, task, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_inject_message(self, client: Lightcone) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.agent.tasks.with_raw_response.inject_message(
                id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_pause(self, client: Lightcone) -> None:
        task = client.agent.tasks.pause(
            "id",
        )
        assert_matches_type(TaskPauseResponse, task, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_pause(self, client: Lightcone) -> None:
        response = client.agent.tasks.with_raw_response.pause(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        task = response.parse()
        assert_matches_type(TaskPauseResponse, task, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_pause(self, client: Lightcone) -> None:
        with client.agent.tasks.with_streaming_response.pause(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            task = response.parse()
            assert_matches_type(TaskPauseResponse, task, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_pause(self, client: Lightcone) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.agent.tasks.with_raw_response.pause(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_resume(self, client: Lightcone) -> None:
        task = client.agent.tasks.resume(
            "id",
        )
        assert_matches_type(TaskResumeResponse, task, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_resume(self, client: Lightcone) -> None:
        response = client.agent.tasks.with_raw_response.resume(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        task = response.parse()
        assert_matches_type(TaskResumeResponse, task, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_resume(self, client: Lightcone) -> None:
        with client.agent.tasks.with_streaming_response.resume(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            task = response.parse()
            assert_matches_type(TaskResumeResponse, task, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_resume(self, client: Lightcone) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.agent.tasks.with_raw_response.resume(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve_status(self, client: Lightcone) -> None:
        task = client.agent.tasks.retrieve_status(
            "id",
        )
        assert_matches_type(TaskRetrieveStatusResponse, task, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve_status(self, client: Lightcone) -> None:
        response = client.agent.tasks.with_raw_response.retrieve_status(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        task = response.parse()
        assert_matches_type(TaskRetrieveStatusResponse, task, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve_status(self, client: Lightcone) -> None:
        with client.agent.tasks.with_streaming_response.retrieve_status(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            task = response.parse()
            assert_matches_type(TaskRetrieveStatusResponse, task, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve_status(self, client: Lightcone) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.agent.tasks.with_raw_response.retrieve_status(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_start(self, client: Lightcone) -> None:
        task = client.agent.tasks.start()
        assert_matches_type(TaskStartResponse, task, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_start_with_all_params(self, client: Lightcone) -> None:
        task = client.agent.tasks.start(
            agent_type="agent_type",
            environment_id="environment_id",
            instruction="instruction",
            kind="desktop",
            max_steps=0,
            model="model",
            persistent=True,
            screenshot_mode="url",
            temperature=0,
            viewport_height=0,
            viewport_width=0,
        )
        assert_matches_type(TaskStartResponse, task, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_start(self, client: Lightcone) -> None:
        response = client.agent.tasks.with_raw_response.start()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        task = response.parse()
        assert_matches_type(TaskStartResponse, task, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_start(self, client: Lightcone) -> None:
        with client.agent.tasks.with_streaming_response.start() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            task = response.parse()
            assert_matches_type(TaskStartResponse, task, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_start_stream(self, client: Lightcone) -> None:
        task_stream = client.agent.tasks.start_stream()
        task_stream.response.close()

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_start_stream_with_all_params(self, client: Lightcone) -> None:
        task_stream = client.agent.tasks.start_stream(
            agent_type="agent_type",
            environment_id="environment_id",
            instruction="instruction",
            kind="desktop",
            max_steps=0,
            model="model",
            persistent=True,
            screenshot_mode="url",
            temperature=0,
            viewport_height=0,
            viewport_width=0,
        )
        task_stream.response.close()

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_start_stream(self, client: Lightcone) -> None:
        response = client.agent.tasks.with_raw_response.start_stream()

        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        stream = response.parse()
        stream.close()

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_start_stream(self, client: Lightcone) -> None:
        with client.agent.tasks.with_streaming_response.start_stream() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            stream = response.parse()
            stream.close()

        assert cast(Any, response.is_closed) is True


class TestAsyncTasks:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_inject_message(self, async_client: AsyncLightcone) -> None:
        task = await async_client.agent.tasks.inject_message(
            id="id",
        )
        assert_matches_type(TaskInjectMessageResponse, task, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_inject_message_with_all_params(self, async_client: AsyncLightcone) -> None:
        task = await async_client.agent.tasks.inject_message(
            id="id",
            message="message",
        )
        assert_matches_type(TaskInjectMessageResponse, task, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_inject_message(self, async_client: AsyncLightcone) -> None:
        response = await async_client.agent.tasks.with_raw_response.inject_message(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        task = await response.parse()
        assert_matches_type(TaskInjectMessageResponse, task, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_inject_message(self, async_client: AsyncLightcone) -> None:
        async with async_client.agent.tasks.with_streaming_response.inject_message(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            task = await response.parse()
            assert_matches_type(TaskInjectMessageResponse, task, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_inject_message(self, async_client: AsyncLightcone) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.agent.tasks.with_raw_response.inject_message(
                id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_pause(self, async_client: AsyncLightcone) -> None:
        task = await async_client.agent.tasks.pause(
            "id",
        )
        assert_matches_type(TaskPauseResponse, task, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_pause(self, async_client: AsyncLightcone) -> None:
        response = await async_client.agent.tasks.with_raw_response.pause(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        task = await response.parse()
        assert_matches_type(TaskPauseResponse, task, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_pause(self, async_client: AsyncLightcone) -> None:
        async with async_client.agent.tasks.with_streaming_response.pause(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            task = await response.parse()
            assert_matches_type(TaskPauseResponse, task, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_pause(self, async_client: AsyncLightcone) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.agent.tasks.with_raw_response.pause(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_resume(self, async_client: AsyncLightcone) -> None:
        task = await async_client.agent.tasks.resume(
            "id",
        )
        assert_matches_type(TaskResumeResponse, task, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_resume(self, async_client: AsyncLightcone) -> None:
        response = await async_client.agent.tasks.with_raw_response.resume(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        task = await response.parse()
        assert_matches_type(TaskResumeResponse, task, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_resume(self, async_client: AsyncLightcone) -> None:
        async with async_client.agent.tasks.with_streaming_response.resume(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            task = await response.parse()
            assert_matches_type(TaskResumeResponse, task, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_resume(self, async_client: AsyncLightcone) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.agent.tasks.with_raw_response.resume(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve_status(self, async_client: AsyncLightcone) -> None:
        task = await async_client.agent.tasks.retrieve_status(
            "id",
        )
        assert_matches_type(TaskRetrieveStatusResponse, task, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve_status(self, async_client: AsyncLightcone) -> None:
        response = await async_client.agent.tasks.with_raw_response.retrieve_status(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        task = await response.parse()
        assert_matches_type(TaskRetrieveStatusResponse, task, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve_status(self, async_client: AsyncLightcone) -> None:
        async with async_client.agent.tasks.with_streaming_response.retrieve_status(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            task = await response.parse()
            assert_matches_type(TaskRetrieveStatusResponse, task, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve_status(self, async_client: AsyncLightcone) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.agent.tasks.with_raw_response.retrieve_status(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_start(self, async_client: AsyncLightcone) -> None:
        task = await async_client.agent.tasks.start()
        assert_matches_type(TaskStartResponse, task, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_start_with_all_params(self, async_client: AsyncLightcone) -> None:
        task = await async_client.agent.tasks.start(
            agent_type="agent_type",
            environment_id="environment_id",
            instruction="instruction",
            kind="desktop",
            max_steps=0,
            model="model",
            persistent=True,
            screenshot_mode="url",
            temperature=0,
            viewport_height=0,
            viewport_width=0,
        )
        assert_matches_type(TaskStartResponse, task, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_start(self, async_client: AsyncLightcone) -> None:
        response = await async_client.agent.tasks.with_raw_response.start()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        task = await response.parse()
        assert_matches_type(TaskStartResponse, task, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_start(self, async_client: AsyncLightcone) -> None:
        async with async_client.agent.tasks.with_streaming_response.start() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            task = await response.parse()
            assert_matches_type(TaskStartResponse, task, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_start_stream(self, async_client: AsyncLightcone) -> None:
        task_stream = await async_client.agent.tasks.start_stream()
        await task_stream.response.aclose()

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_start_stream_with_all_params(self, async_client: AsyncLightcone) -> None:
        task_stream = await async_client.agent.tasks.start_stream(
            agent_type="agent_type",
            environment_id="environment_id",
            instruction="instruction",
            kind="desktop",
            max_steps=0,
            model="model",
            persistent=True,
            screenshot_mode="url",
            temperature=0,
            viewport_height=0,
            viewport_width=0,
        )
        await task_stream.response.aclose()

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_start_stream(self, async_client: AsyncLightcone) -> None:
        response = await async_client.agent.tasks.with_raw_response.start_stream()

        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        stream = await response.parse()
        await stream.close()

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_start_stream(self, async_client: AsyncLightcone) -> None:
        async with async_client.agent.tasks.with_streaming_response.start_stream() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            stream = await response.parse()
            await stream.close()

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["ChatCompletionContentPartImageParam", "ImageURL"]


class ImageURL(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    url: Required[str]

    detail: Literal["auto", "low", "high"]


class ChatCompletionContentPartImageParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """Learn about [image inputs](https://platform.openai.com/docs/guides/vision)."""

    image_url: Required[ImageURL]

    type: Required[Literal["image_url"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["McpApprovalRequestParam"]


class McpApprovalRequestParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A request for human approval of a tool invocation."""

    id: Required[str]

    arguments: Required[str]

    name: Required[str]

    server_label: Required[str]

    type: Required[Literal["mcp_approval_request"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .file_param import FileParam as FileParam
from .action_result import ActionResult as ActionResult
from .logprob_param import LogprobParam as LogprobParam
from .message_param import MessageParam as MessageParam
from .summary_param import SummaryParam as SummaryParam
from .action_drag_param import ActionDragParam as ActionDragParam
from .action_find_param import ActionFindParam as ActionFindParam
from .action_move_param import ActionMoveParam as ActionMoveParam
from .action_type_param import ActionTypeParam as ActionTypeParam
from .action_wait_param import ActionWaitParam as ActionWaitParam
from .computer_response import ComputerResponse as ComputerResponse
from .output_logs_param import OutputLogsParam as OutputLogsParam
from .action_click_param import ActionClickParam as ActionClickParam
from .output_image_param import OutputImageParam as OutputImageParam
from .action_scroll_param import ActionScrollParam as ActionScrollParam
from .computer_drag_params import ComputerDragParams as ComputerDragParams
from .computer_html_params import ComputerHTMLParams as ComputerHTMLParams
from .computer_list_params import ComputerListParams as ComputerListParams
from .computer_type_params import ComputerTypeParams as ComputerTypeParams
from .action_keypress_param import ActionKeypressParam as ActionKeypressParam
from .computer_action_param import ComputerActionParam as ComputerActionParam
from .computer_batch_params import ComputerBatchParams as ComputerBatchParams
from .computer_click_params import ComputerClickParams as ComputerClickParams
from .computer_debug_params import ComputerDebugParams as ComputerDebugParams
from .action_open_page_param import ActionOpenPageParam as ActionOpenPageParam
from .computer_create_params import ComputerCreateParams as ComputerCreateParams
from .computer_hotkey_params import ComputerHotkeyParams as ComputerHotkeyParams
from .computer_key_up_params import ComputerKeyUpParams as ComputerKeyUpParams
from .computer_list_response import ComputerListResponse as ComputerListResponse
from .computer_scroll_params import ComputerScrollParams as ComputerScrollParams
from .response_create_params import ResponseCreateParams as ResponseCreateParams
from .action_screenshot_param import ActionScreenshotParam as ActionScreenshotParam
from .comparison_filter_param import ComparisonFilterParam as ComparisonFilterParam
from .computer_batch_response import ComputerBatchResponse as ComputerBatchResponse
from .computer_execute_params import ComputerExecuteParams as ComputerExecuteParams
from .computer_key_down_params import ComputerKeyDownParams as ComputerKeyDownParams
from .computer_mouse_up_params import ComputerMouseUpParams as ComputerMouseUpParams
from .computer_navigate_params import ComputerNavigateParams as ComputerNavigateParams
from .computer_viewport_params import ComputerViewportParams as ComputerViewportParams
from .response_retrieve_params import ResponseRetrieveParams as ResponseRetrieveParams
from .action_double_click_param import ActionDoubleClickParam as ActionDoubleClickParam
from .response_input_file_param import ResponseInputFileParam as ResponseInputFileParam
from .response_input_text_param import ResponseInputTextParam as ResponseInputTextParam
from .action_search_source_param import ActionSearchSourceParam as ActionSearchSourceParam
from .annotation_file_path_param import AnnotationFilePathParam as AnnotationFilePathParam
from .computer_mouse_down_params import ComputerMouseDownParams as ComputerMouseDownParams
from .computer_screenshot_params import ComputerScreenshotParams as ComputerScreenshotParams
from .mcp_approval_request_param import McpApprovalRequestParam as McpApprovalRequestParam
from .response_input_image_param import ResponseInputImageParam as ResponseInputImageParam
from .computer_keepalive_response import ComputerKeepaliveResponse as ComputerKeepaliveResponse
from .computer_right_click_params import ComputerRightClickParams as ComputerRightClickParams
from .computer_change_proxy_params import ComputerChangeProxyParams as ComputerChangeProxyParams
from .computer_double_click_params import ComputerDoubleClickParams as ComputerDoubleClickParams
from .annotation_url_citation_param import AnnotationURLCitationParam as AnnotationURLCitationParam
from .chat_create_completion_params import ChatCreateCompletionParams as ChatCreateCompletionParams
from .annotation_file_citation_param import AnnotationFileCitationParam as AnnotationFileCitationParam
from .computer_retrieve_status_response import ComputerRetrieveStatusResponse as ComputerRetrieveStatusResponse
from .chat_completion_content_part_text_param import (
    ChatCompletionContentPartTextParam as ChatCompletionContentPartTextParam,
)
from .annotation_container_file_citation_param import (
    AnnotationContainerFileCitationParam as AnnotationContainerFileCitationParam,
)
from .chat_completion_content_part_audio_param import (
    ChatCompletionContentPartAudioParam as ChatCompletionContentPartAudioParam,
)
from .chat_completion_content_part_image_param import (
    ChatCompletionContentPartImageParam as ChatCompletionContentPartImageParam,
)
from .chat_completion_content_part_refusal_param import (
    ChatCompletionContentPartRefusalParam as ChatCompletionContentPartRefusalParam,
)
from .chat_completion_message_function_tool_call_param import (
    ChatCompletionMessageFunctionToolCallParam as ChatCompletionMessageFunctionToolCallParam,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["AnnotationContainerFileCitationParam"]


class AnnotationContainerFileCitationParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """A citation for a container file used to generate a model response."""

    container_id: Required[str]

    end_index: Required[int]

    file_id: Required[str]

    filename: Required[str]

    start_index: Required[int]

    type: Required[Literal["container_file_citation"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["ComputerKeyUpParams"]


class ComputerKeyUpParams(TypedDict, total=False):
    key: str
    """Key name to release. Case-insensitive. Examples: "shift", "ctrl", "a", "Enter" """

    tab_id: str
    """Optional tab ID for browser sessions (ignored for desktop sessions)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["MessageParam", "Author"]


class Author(TypedDict, total=False):
    role: Required[Literal["user", "assistant", "system", "developer", "tool"]]
    """The role of a message author (mirrors `chat::Role`)."""

    name: Optional[str]


class MessageParam(TypedDict, total=False):
    author: Required[Author]

    channel: Optional[str]

    content: Iterable[object]

    content_type: Optional[str]

    recipient: Optional[str]



"""Convenience wrapper around the Lightcone computers API."""

from __future__ import annotations

from typing import TYPE_CHECKING, Any, Type, Iterable, Optional, cast
from typing_extensions import override

if TYPE_CHECKING:
    from .._client import Lightcone
    from ..types.action_result import ActionResult
    from ..types.computer_response import ComputerResponse
    from ..types.computer_action_param import ComputerActionParam
    from ..types.computer_batch_response import ComputerBatchResponse
    from ..types.computer_keepalive_response import ComputerKeepaliveResponse

__all__ = ["ComputerResource", "ComputerSession"]


class ComputerResource:
    """
    Resource for creating and managing computer sessions.

    Accessed as ``client.computer`` on a :class:`Lightcone` client.

    Example::

        from tzafon import Lightcone

        client = Lightcone()
        with client.computer.create(kind="browser") as computer:
            computer.navigate("https://example.com")
            computer.click(100, 200)
            result = computer.screenshot()
    """

    def __init__(self, client: Lightcone) -> None:
        self._client = client

    def create(self, kind: str = "browser", **kwargs: Any) -> ComputerSession:
        """
        Create a new computer session.

        Args:
            kind: Session type — ``"browser"`` or ``"desktop"``
            **kwargs: Additional arguments passed to the API (e.g.
                ``timeout_seconds``, ``inactivity_timeout_seconds``,
                ``persistent``, ``display``, ``stealth``).

        Returns:
            A :class:`ComputerSession` bound to the new session ID.
        """
        response = self._client.computers.create(kind=kind, **kwargs)
        assert response.id is not None
        return ComputerSession(self._client, response.id)


class ComputerSession:
    """
    High-level wrapper that binds a computer ID to a client.

    Provides the same methods as the previous ``computer-python`` SDK,
    with automatic ID management and context-manager cleanup.

    Example::

        with client.computer.create(kind="browser") as computer:
            computer.navigate("https://example.com")
            computer.click(100, 200)
            result = computer.screenshot()
            url = computer.get_screenshot_url(result)
            # automatically terminated on exit
    """

    def __init__(self, client: Lightcone, computer_id: str) -> None:
        self._client = client
        self.id = computer_id

    # ── Navigation ────────────────────────────────────────────────

    def navigate(self, url: str) -> ActionResult:
        """Navigate to a URL."""
        return self._client.computers.navigate(self.id, url=url)

    # ── Mouse ─────────────────────────────────────────────────────

    def click(self, x: float, y: float) -> ActionResult:
        """Click at coordinates."""
        return self._client.computers.click(self.id, x=x, y=y)

    def double_click(self, x: float, y: float) -> ActionResult:
        """Double-click at coordinates."""
        return self._client.computers.double_click(self.id, x=x, y=y)

    def right_click(self, x: float, y: float) -> ActionResult:
        """Right-click at coordinates."""
        return self._client.computers.right_click(self.id, x=x, y=y)

    def drag(self, x1: float, y1: float, x2: float, y2: float) -> ActionResult:
        """Drag from (x1, y1) to (x2, y2)."""
        return self._client.computers.drag(self.id, x1=x1, y1=y1, x2=x2, y2=y2)

    def mouse_down(self, x: float, y: float) -> ActionResult:
        """Press mouse button at coordinates."""
        return self._client.computers.mouse_down(self.id, x=x, y=y)

    def mouse_up(self, x: float, y: float) -> ActionResult:
        """Release mouse button at coordinates."""
        return self._client.computers.mouse_up(self.id, x=x, y=y)

    # ── Keyboard ──────────────────────────────────────────────────

    def type(self, text: str) -> ActionResult:
        """Type text into the focused element."""
        return self._client.computers.type(self.id, text=text)

    def hotkey(self, *keys: str | list[str] | tuple[str, ...]) -> ActionResult:
        """
        Press a keyboard shortcut.

        Accepts varargs or a single list/tuple::

            computer.hotkey("Control", "c")
            computer.hotkey(["Control", "v"])
        """
        keys_list: list[str]
        if len(keys) == 1 and isinstance(keys[0], (list, tuple)):
            keys_list = list(keys[0])
        else:
            keys_list = cast(list[str], list(keys))
        return self._client.computers.hotkey(self.id, keys=keys_list)

    def key_down(self, key: str) -> ActionResult:
        """Press and hold a key."""
        return self._client.computers.key_down(self.id, key=key)

    def key_up(self, key: str) -> ActionResult:
        """Release a held key."""
        return self._client.computers.key_up(self.id, key=key)

    # ── Scrolling & Viewport ──────────────────────────────────────

    def scroll(self, dx: float = 0, dy: float = 0, x: float = 0, y: float = 0) -> ActionResult:
        """
        Scroll the viewport.

        Args:
            dx: Horizontal scroll delta (positive = right)
            dy: Vertical scroll delta (positive = down)
            x: Scroll origin X coordinate
            y: Scroll origin Y coordinate
        """
        return self._client.computers.scroll(self.id, dx=dx, dy=dy, x=x, y=y)

    def set_viewport(self, width: int, height: int, scale_factor: float = 1.0) -> ActionResult:
        """Change viewport dimensions."""
        return self._client.computers.viewport(self.id, width=width, height=height, scale_factor=scale_factor)

    # ── Content ───────────────────────────────────────────────────

    def screenshot(self, base64: bool = False) -> ActionResult:
        """
        Capture a screenshot.

        Use ``get_screenshot_url(result)`` to extract the URL from the result.
        """
        return self._client.computers.screenshot(self.id, base64=base64)

    def html(self, auto_detect_encoding: bool = False) -> ActionResult:
        """
        Get page HTML content.

        Use ``get_html_content(result)`` to extract the HTML from the result.
        """
        return self._client.computers.html(self.id, auto_detect_encoding=auto_detect_encoding)

    # ── Shell / Debug ─────────────────────────────────────────────

    def debug(self, command: str, timeout_seconds: int = 120, max_output_length: int = 65536) -> ActionResult:
        """
        Execute a debug/shell command.

        Use ``get_debug_response(result)`` to extract the output from the result.
        """
        return self._client.computers.debug(
            self.id, command=command, timeout_seconds=timeout_seconds, max_output_length=max_output_length
        )

    # ── Timing ────────────────────────────────────────────────────

    def wait(self, seconds: float) -> ActionResult:
        """
        Wait for a duration on the remote computer.

        Args:
            seconds: Seconds to wait (e.g. 0.5 for 500ms)
        """
        ms = int(seconds * 1000)
        return self._client.computers.execute(self.id, action={"type": "wait", "ms": ms})

    # ── Generic / Batch ───────────────────────────────────────────

    def execute(self, action: ComputerActionParam) -> ActionResult:
        """
        Execute a single generic action.

        Useful for actions without dedicated methods (tab ops, etc.)::

            computer.execute({"type": "new_tab", "url": "https://example.com"})
            computer.execute({"type": "switch_tab", "tab_id": "tab_123"})
        """
        return self._client.computers.execute(self.id, action=action)

    def batch(self, actions: Iterable[ComputerActionParam]) -> ComputerBatchResponse:
        """
        Execute multiple actions in sequence with a single API call.

        Stops on first error::

            computer.batch([
                {"type": "go_to_url", "url": "https://example.com"},
                {"type": "wait", "ms": 2000},
                {"type": "click", "x": 100, "y": 200},
                {"type": "screenshot"},
            ])
        """
        return self._client.computers.batch(self.id, actions=actions)

    # ── Session management ────────────────────────────────────────

    def keep_alive(self) -> ComputerKeepaliveResponse:
        """Extend session timeout to prevent automatic termination."""
        return self._client.computers.keepalive(self.id)

    def retrieve(self) -> ComputerResponse:
        """Get current session status and metadata."""
        return self._client.computers.retrieve(self.id)

    def terminate(self) -> None:
        """Terminate the session and clean up resources."""
        self._client.computers.delete(self.id)

    # ── Result helpers ────────────────────────────────────────────

    @staticmethod
    def get_screenshot_url(result: ActionResult) -> Optional[str]:
        """Extract screenshot URL from an ActionResult."""
        if result.result:
            return result.result.get("screenshot_url")  # type: ignore[return-value]
        return None

    @staticmethod
    def get_html_content(result: ActionResult) -> Optional[str]:
        """Extract HTML content from an ActionResult."""
        if result.result:
            return result.result.get("html_content")  # type: ignore[return-value]
        return None

    @staticmethod
    def get_debug_response(result: ActionResult) -> Optional[str]:
        """Extract debug command output from an ActionResult."""
        if result.result:
            return result.result.get("debug_response")  # type: ignore[return-value]
        return None

    # ── Context manager ───────────────────────────────────────────

    def __enter__(self) -> ComputerSession:
        return self

    def __exit__(
        self,
        exc_type: Type[BaseException] | None,
        exc_val: BaseException | None,
        exc_tb: object,
    ) -> None:
        self.terminate()

    @override
    def __repr__(self) -> str:
        return f"ComputerSession(id={self.id!r})"



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["ActionFindParam"]


class ActionFindParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """Action type "find": Searches for a pattern within a loaded page."""

    pattern: Required[str]

    type: Required[Literal["find"]]

    url: Required[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["ChatCompletionContentPartAudioParam", "InputAudio"]


class InputAudio(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    data: Required[str]

    format: Required[Literal["wav", "mp3"]]


class ChatCompletionContentPartAudioParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """Learn about [audio inputs](https://platform.openai.com/docs/guides/audio)."""

    input_audio: Required[InputAudio]

    type: Required[Literal["input_audio"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["OutputLogsParam"]


class OutputLogsParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """The logs output from the code interpreter."""

    logs: Required[str]

    type: Required[Literal["logs"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["ChatCompletionContentPartTextParam"]


class ChatCompletionContentPartTextParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """
    Learn about [text inputs](https://platform.openai.com/docs/guides/text-generation).
    """

    text: Required[str]

    type: Required[Literal["text"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["OutputImageParam"]


class OutputImageParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """The image output from the code interpreter."""

    type: Required[Literal["image"]]

    url: Required[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .task_start_params import TaskStartParams as TaskStartParams
from .task_pause_response import TaskPauseResponse as TaskPauseResponse
from .task_start_response import TaskStartResponse as TaskStartResponse
from .task_resume_response import TaskResumeResponse as TaskResumeResponse
from .task_start_stream_params import TaskStartStreamParams as TaskStartStreamParams
from .task_inject_message_params import TaskInjectMessageParams as TaskInjectMessageParams
from .task_start_stream_response import TaskStartStreamResponse as TaskStartStreamResponse
from .task_inject_message_response import TaskInjectMessageResponse as TaskInjectMessageResponse
from .task_retrieve_status_response import TaskRetrieveStatusResponse as TaskRetrieveStatusResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["ComputerListParams"]


class ComputerListParams(TypedDict, total=False):
    type: Literal["live", "persistent"]
    """Session type filter"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, TypedDict

from .._types import SequenceNotStr

__all__ = ["ComparisonFilterParam"]


class ComparisonFilterParam(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """
    A filter used to compare a specified attribute key to a given value using a defined comparison operation.
    """

    key: Required[str]

    type: Required[Literal["eq", "ne", "gt", "gte", "lt", "lte"]]

    value: Required[Union[str, float, bool, SequenceNotStr[Union[str, float]]]]
